/*
 * Copyright (c) 1993-1995 by Argonaut Technologies Limited. All rights reserved.
 *
 * $Id: compiler.h 1.1 1996/09/12 14:02:37 NIKH Exp $
 * $Locker: $
 *
 * Misc host compiler configuration (types & special declarators etc.)
 *
 *
 * PS	9/3/95	SE	Initial mods for psyq GCC
 *
 */
#ifndef _COMPILER_H_
#define _COMPILER_H_

#ifdef __cplusplus
	extern "C" {
#endif

#ifdef __PS__

#define offsetof(typ,id) (size_t)&(((typ*)0)->id)
extern double	sqrt( double __x );

#endif

/*
 * Fixed bitsize integers
 */
typedef signed long br_int_32;
typedef unsigned long br_uint_32;

typedef signed short br_int_16;
typedef unsigned short br_uint_16;

typedef signed char br_int_8;
typedef unsigned char br_uint_8;

/*
 * Generic size type (in case target environment does not have size_t)
 */
typedef unsigned int br_size_t;

/*
 * Boolean type
 */
typedef int br_boolean;

#define BR_TRUE         1
#define BR_FALSE        0

#define BR_BOOLEAN(a)   ((a) != 0)

/**
 ** Compiler specific declarations
 **
 *
 * BR_PUBLIC_ENTRY are the declarators for public entry points into the library
 *
 * BR_CALLBACK are the declarators for function pointers to callbacks
 *
 * BR_ASM_DATA is the declarator for data that is shared with ASM files
 *
 * BR_ASM_CALL is the declarator for function declarations that are ASM functions
 * called from C
 *
 * BR_ASM_CALLBACK is the declarator for function definitions that are C functions
 * called from ASM
 *
 * BR_SUFFIX_HOST is a string used for banners/titles etc
 *
 * br_framebuffer_ptr is a type used for pointing at the pixels of a frame buffer
 *
 * BR_HAS_FAR is !0 if language supports __far pointers
 */

/*
 * GNU C
 */
#if defined (__GNUC__)
#define MEMAVL	0

#define BR_PUBLIC_ENTRY
#define BR_CALLBACK

#define BR_ASM_DATA
#define BR_ASM_CALL
#define BR_ASM_DATA_EXTRA(x)
#define BR_ASM_CALL_EXTRA(x)
#define BR_ASM_CALLBACK

#define BR_SUFFIX_HOST "-GCC"

#define BR_HAS_FAR	0

typedef br_uint_8 * br_framebuffer_ptr;

#endif

#if DEBUG
#	define BR_SUFFIX_DEBUG "-DEBUG"
#else
#	define BR_SUFFIX_DEBUG ""
#endif

/*
 * Macros for producing banners & copyright messages
 */
#if DEBUG

#define BR_BANNER(title,year,revision)do {\
	printf("%s %s Copyright (C) %s by Argonaut Technologies Limited\n",title,revision+10,year);\
} while(0);

#else

#define BR_BANNER(title,year,revision)

#endif

/*
 * Useful macro for sizing an array
 */
#define BR_ASIZE(a) (sizeof(a)/sizeof((a)[0]))

#ifdef __cplusplus
	};
#endif
#endif


