/*
 * Copyright (c) 1993-1995 by Argonaut Technologies Limited. All rights reserved.
 *
 * $Id: ps.h 1.1 1996/09/12 14:02:44 NIKH Exp $
 * $Locker: $
 *
 * Definitons for PS
 */
#ifndef _PS_H_
#define _PS_H_

#ifdef __cplusplus
extern "C" {
#endif

typedef struct br_vector3s {
    
	br_int_16	v[3];
	br_int_16	_pad;
	
} br_vector3s;

typedef struct br_vector2s {

	br_int_16	v[2];
	
} br_vector2s;

typedef struct br_vector4b {
    
    	br_uint_8	v[4];
	
} br_vector4b;

typedef struct br_vector3b {

	br_uint_8	v[3];

} br_vector3b;

typedef struct br_vector2b {

	br_uint_8	v[2];
    
} br_vector2b;

typedef struct brps_prim_tag {

	br_uint_32	addr:24;
	br_uint_32	len:8;
	
} brps_prim_tag;

/*
 * rectangle
 */
typedef struct brps_rectangle {

	br_uint_16	x,y;
	br_uint_16	w,h;
    
} brps_rectangle;

/*
 * GPU rendering primitives
 */

/*
 * Flat triangle
 */
typedef struct brps_prim_poly_f3 {

	brps_prim_tag	tag;
	
	br_vector4b	colour0;
	
	br_vector2s	v0;
	br_vector2s	v1;
	br_vector2s	v2;
	
} brps_prim_poly_f3;

/*
 * Flat quadrangle
 */
typedef struct brps_prim_poly_f4 {

	brps_prim_tag	tag;
	
	br_vector4b	colour0;

	br_vector2s	v0;
	br_vector2s	v1;
	br_vector2s	v2;
	br_vector2s	v3;
    
} brps_prim_poly_f4;

/*
 * Flat textured triangle
 */
typedef struct brps_prim_poly_ft3 {

	brps_prim_tag	tag;

	br_vector4b	colour0;
	
	br_vector2s	v0;
	br_vector2b	map0;
	
	br_uint_16	clut;

	br_vector2s	v1;
	br_vector2b	map1;

	br_uint_16	tpage;

	br_vector2s	v2;
	br_vector2b	map2;

	br_uint_16	_pad;
	
} brps_prim_poly_ft3;

/*
 * Flat textured quadrangle
 */
typedef struct brps_prim_poly_ft4 {

	brps_prim_tag	tag;

	br_vector4b	colour0;
	
	br_vector2s	v0;
	br_vector2b	map0;
	
	br_uint_16	clut;

	br_vector2s	v1;
	br_vector2b	map1;

	br_uint_16	tpage;

	br_vector2s	v2;
	br_vector2b	map2;

	br_uint_16	_pad;

	br_vector2s	v3;
	br_vector2b	map3;

	br_uint_16	_pad1;
	
} brps_prim_poly_ft4;

/*
 * Gouraud triangle
 */
typedef struct brps_prim_poly_g3 {

	brps_prim_tag	tag;

	br_vector4b	colour0;
	br_vector2s	v0;
	
	br_vector4b	colour1;
	br_vector2s	v1;

	br_vector4b	colour2;
	br_vector2s	v2;
    
} brps_prim_poly_g3;

/*
 * Gouraud quadrangle
 */
typedef struct brps_prim_poly_g4 {

	brps_prim_tag	tag;

	br_vector4b	colour0;
	br_vector2s	v0;

	br_vector4b	colour1;
	br_vector2s	v1;

	br_vector4b	colour2;
	br_vector2s	v2;

	br_vector4b	colour3;
	br_vector2s	v3;
    
} brps_prim_poly_g4;

/*
 * Gouraud textured triangle
 */
typedef struct brps_prim_poly_gt3 {

	brps_prim_tag	tag;

	br_vector4b	colour0;
	br_vector2s	v0;
	br_vector2b	map0;

	br_uint_16	clut;

	br_vector4b	colour1;
	br_vector2s	v1;
	br_vector2b	map1;

	br_uint_16	tpage;

	br_vector4b	colour2;
	br_vector2s	v2;
	br_vector2b	map2;

	br_uint_16	_pad;
	    
} brps_prim_poly_gt3;

/*
 * Gouraud textured quadrangle
 */
typedef struct brps_prim_poly_gt4 {

	brps_prim_tag	tag;

	br_vector4b	colour0;
	br_vector2s	v0;
	br_vector2b	map0;

	br_uint_16	clut;

	br_vector4b	colour1;
	br_vector2s	v1;
	br_vector2b	map1;

	br_uint_16	tpage;

	br_vector4b	colour2;
	br_vector2s	v2;
	br_vector2b	map2;

	br_uint_16	_pad;

	br_vector4b	colour3;
	br_vector2s	v3;
	br_vector2b	map3;

	br_uint_16	_pad1;

} brps_prim_poly_gt4;

/*
 * Unconnected flat line
 */
typedef struct brps_prim_line_f2 {

	brps_prim_tag	tag;

	br_vector4b	colour0;

	br_vector2s	v0;
	br_vector2s	v1;
	    
} brps_prim_line_f2;

/*
 * Unconnected gouraud line
 */
typedef struct brps_prim_line_g2 {

	brps_prim_tag	tag;

	br_vector4b	colour0;
	br_vector2s	v0;

	br_vector4b	colour1;
	br_vector2s	v1;
	    
} brps_prim_line_g2;

/*
 * 2 connected flat lines
 */
typedef struct brps_prim_line_f3 {

	brps_prim_tag	tag;

	br_vector4b	colour0;

	br_vector2s	v0;
	br_vector2s	v1;
	br_vector2s	v2;

	br_uint_32	_pad;
	    
} brps_prim_line_f3;

/*
 * 2 connected gouraud lines
 */
typedef struct brps_prim_line_g3 {

	brps_prim_tag	tag;

	br_vector4b	colour0;
	br_vector2s	v0;

	br_vector4b	colour1;
	br_vector2s	v1;

	br_vector4b	colour2;
	br_vector2s	v2;

	br_uint_32	_pad;

} brps_prim_line_g3;

/*
 * 3 connected flat lines
 */
typedef struct brps_prim_line_f4 {

	brps_prim_tag	tag;

	br_vector4b	colour0;
	
	br_vector2s	v0;
	br_vector2s	v1;
	br_vector2s	v2;
	br_vector2s	v3;

	br_uint_32	_pad;
	
} brps_prim_line_f4;

/*
 * 3 connected gouraud lines
 */
typedef struct brps_prim_line_g4 {

	brps_prim_tag	tag;

	br_vector4b	colour0;
	br_vector2s	v0;

	br_vector4b	colour1;
	br_vector2s	v1;

	br_vector4b	colour2;
	br_vector2s	v2;

	br_vector4b	colour3;
	br_vector2s	v3;

	br_uint_32	_pad;
	
} brps_prim_line_g4;

/*
 * free size sprite
 */
typedef struct brps_prim_sprite {

	brps_prim_tag	tag;
	
	br_vector4b	colour0;
	
	br_vector2s	v0;
	br_vector2b	map0;
	
	br_uint_16	clut;
	
	br_int_16	w,h;
    
} brps_prim_sprite;

/*
 * 16x16 sprite
 */
typedef struct brps_prim_sprite_16 {

	brps_prim_tag	tag;

	br_vector4b	colour0;

	br_vector2s	v0;
	br_vector2b	map0;

	br_uint_16	clut;
    
} brps_prim_sprite_16;

/*
 * 8x8 sprite
 */
typedef struct brps_prim_sprite_8 {

	brps_prim_tag	tag;

	br_vector4b	colour0;

	br_vector2s	v0;
	br_vector2b	map0;

	br_uint_16	clut;
    
} brps_prim_sprite_8;

/*
 * free size tile
 */
typedef struct brps_prim_tile {

	brps_prim_tag	tag;

	br_vector4b	colour0;

	br_vector2s	v0;
	br_int_16	w,h;
    
} brps_prim_tile;

/*
 * 16x16 tile
 */
typedef struct brps_prim_tile_16 {

	brps_prim_tag	tag;

	br_vector4b	colour0;

	br_vector2s	v0;
    
} brps_prim_tile_16;

/*
 * 8x8 tile
 */
typedef struct brps_prim_tile_8 {

	brps_prim_tag	tag;

	br_vector4b	colour0;

	br_vector2s	v0;
    
} brps_prim_tile_8;

/*
 * 1x1 tile
 */
typedef struct brps_prim_tile_1 {

	brps_prim_tag	tag;

	br_vector4b	colour0;

	br_vector2s	v0;
	   
} brps_prim_tile_1;


/*
 * Environment
 */
typedef struct brps_prim_environment {

	brps_prim_tag	tag;
	br_uint_32	code[15];
    
} brps_prim_environment;

/*
 * Drawing environment
 */
typedef struct br_draw_environment {

	brps_rectangle	clip;		/* clip area				*/
	br_vector2s	offset;		/* drawing offset			*/
	brps_rectangle	texture_window;	/* texture window!			*/
	br_uint_16	tpage;		/* texture page				*/
	br_uint_8	dither;		/* 0:off, 1:on				*/
	br_uint_8	display_draw;	/* draw on display area, 0:off, 1:on)	*/
	br_uint_8	clear_screen;	/* enable auto-clear			*/
	br_uint_8	r,g,b;		/* background colour			*/
	brps_prim_environment dr_env;	/* rendering primitive (reserved)	*/
		
} brps_draw_environment;

/*
 * Display environment
 */
typedef struct br_display_environment {

	brps_rectangle	display;	/* display area				*/
	brps_rectangle	screen;		/* display start point			*/
	br_uint_8	interlace;	/* 0:off, 1:on				*/
	br_uint_8	rgb24;		/* 24 bit mode				*/
	br_uint_8	type;		/* 0:PAL, 1:NTSC			*/
	br_uint_8	_pad;		/* reserved				*/
    
} brps_display_environment;

/*
 * Special drawing primitives
 */
 
/*
 * Block fill
 */
typedef struct brps_prim_block_fill {

	brps_prim_tag	tag;

	br_vector4b	colour0;
	br_vector2s	v0;
	br_int_16	w,h;
    
} brps_prim_block_fill;

/*
 * Drawing mode
 */
typedef struct brps_prim_draw_mode {

	brps_prim_tag	tag;
	br_uint_32	code[2];
    
} brps_prim_draw_mode;

/*
 * Priority
 */
typedef struct brps_prim_priority {

	brps_prim_tag	tag;
	br_uint_32	code[2];
    
} brps_prim_priority;

/*
 * Texture Window
 */
typedef struct brps_prim_texture_window {

	brps_prim_tag	tag;
	br_uint_32	code[2];
    
} brps_prim_texture_window;

/*
 * Drawing area
 */
typedef struct brps_prim_draw_area {

	brps_prim_tag	tag;
	br_uint_32	code[2];
	    
} brps_prim_draw_area;

/*
 * Drawing offset
 */
typedef struct brps_prim_draw_offset {

	brps_prim_tag	tag;
	br_uint_32	code[2];
    
} brps_prim_draw_offset;

/*
 * Move image
 */
typedef struct brps_prim_move_image {

	brps_prim_tag	tag;
	
	br_uint_32	code[2];

	br_vector2s	v0;
	br_vector2s	v1;
	br_int_16	w,h;
    
} brps_prim_move_image;

/*
 * BRender custom primitives
 */
typedef struct brps_prim_poly_wft3 {

	void *p;
	brps_prim_texture_window twin;
	brps_prim_poly_ft3 poly;

    
} brps_prim_poly_wft3;

typedef struct brps_prim_poly_wgt3 {

	void *p;
	brps_prim_texture_window twin;
	brps_prim_poly_gt3 poly;
    
} brps_prim_poly_wgt3;

typedef struct brps_prim_poly_mg3 {

	void *p;
	brps_prim_draw_mode mode;
	brps_prim_poly_g3 poly;
    
} brps_prim_poly_mg3;

typedef struct brps_prim_poly_mf3 {

	void *p;
	brps_prim_draw_mode mode;
	brps_prim_poly_f3 poly;
    
} brps_prim_poly_mf3;

typedef struct brps_prim_poly_mgt3 {

	void *p;
	brps_prim_draw_mode mode;
	brps_prim_poly_gt3 poly;
	
} brps_prim_poly_mgt3;

typedef struct brps_prim_poly_mft3 {

	void *p;
	brps_prim_draw_mode mode;
	brps_prim_poly_ft3 poly;
    
} brps_prim_poly_mft3;

/*
 * Macros for libgpu.h
 */
#define BrPSRectangleSet(r, _x, _y, _w, _h) 							\
	(r)->x = (_x), (r)->y = (_y), (r)->w = (_w), (r)->h = (_h)

/*
 *  set attributes (abr is set from material)
 */
#define BrPixelmapTPageSet(p, _tp) 								\
	((p)->tpage = GetTPage((_tp), 0, (p)->base_x, (p)->base_y))

#define BrPixelmapClutSet(p, _x, _y) 								\
	((p)->clut = GetClut(_x, _y))

#define BrPSTPageABRGet(_abr)									\
		((GetGraphType()==1||GetGraphType()==2)?((_abr)&0x3)<<7:((_abr)&0x3)<<5)


/*
 * set primitive colours
 */
#define BrPSPrimColour0Set(p, _r, _g, _b) 							\
	(p)->colour0.v[0] = (_r), (p)->colour0.v[1] = (_g), (p)->colour0.v[2] = (_b)

#define BrPSPrimColour1Set(p, _r, _g, _b) 							\
	(p)->colour1.v[0] = (_r), (p)->colour1.v[1] = (_g), (p)->colour1.v[2] = (_b)
	
#define BrPSPrimColour2Set(p, _r, _g, _b) 							\
	(p)->colour2.v[0] = (_r), (p)->colour2.v[1] = (_g), (p)->colour2.v[2] = (_b)
	
#define BrPSPrimColour3Set(p, _r, _g, _b) 							\
	(p)->colour3.v[0] = (_r), (p)->colour3.v[1] = (_g), (p)->colour3.v[2] = (_b)

/*
 * set vertex coordinates
 */
#define BrPSPrimVertex0Set(p, _x, _y) 							\
	(p)->v0.v[0] = (_x), (p)->v0.v[1] = (_y)

#define BrPSPrimVertex1Set(p, _x, _y) 							\
	(p)->v1.v[0] = (_x), (p)->v1.v[1] = (_y)

#define BrPSPrimVertex2Set(p, _x, _y) 							\
	(p)->v2.v[0] = (_x), (p)->v2.v[1] = (_y)

#define BrPSPrimVertex3Set(p, _x, _y) 							\
	(p)->v3.v[0] = (_x), (p)->v3.v[1] = (_y)

/*
 * set primitive width and height
 */
#define BrPSPrimWHSet(p, _w, _h) 								\
	(p)->w = (_w), (p)->h = (_h)

/*
 * Set mapping coordinates
 */
#define BrPSPrimMap0Set(p, _u, _v) 							\
	(p)->map0.v[0] = (_u), (p)->map0.v[1] = (_v)

#define BrPSPrimMap3Set(p, _u0, _v0, _u1, _v1, _u2, _v2) 					\
	(p)->map0.v[0] = (_u0), (p)->map0.v[1] = (_v0),					\
	(p)->map1.v[0] = (_u1), (p)->map1.v[1] = (_v1),					\
	(p)->map2.v[0] = (_u2), (p)->map2.v[1] = (_v2)
	
#define BrPSPrimMap4Set(p, _u0, _v0, _u1, _v1, _u2, _v2, _u3, _v3)				\
	(p)->map0.v[0] = (_u0), (p)->map0.v[1] = (_v0),						\
	(p)->map1.v[0] = (_u1), (p)->map1.v[1] = (_v1),						\
	(p)->map2.v[0] = (_u2), (p)->map2.v[1] = (_v2),						\
	(p)->map3.v[0] = (_u3), (p)->map3.v[1] = (_v3)

#define BrPSPrimMapWHSet(p, _u, _v, _w, _h) 							\
	(p)->map0.v[0] = (_u), (p)->map0.v[1] = (_v),						\
	(p)->w = (_w), (p)->h = (_h)


/*
 * primitive macros
 */
#define BrPSPrimLenSet(_p, len)		setlen((_p), (len))
#define BrPSPrimAddrSet(_p, addr)		setaddr((_p), (addr))
#define BrPSPrimCodeSet(_p, code)		setcode((_p), (code))

#define BrPSPrimLenGet(_p)		getlen((_p))
#define BrPSPrimAddrGet(_p)		getaddr((_p))
#define BrPSPrimCodeGet(_p)		getcode((_p))

#define BrPSPrimNext(_p)			nextPrim((_p))
#define BrPSPrimIsEnd(_p)			isendprim((_p))

#define BrPSPrimAdd(_ot, _p)		addPrim((_ot), (_p))
#define BrPSPrimCat(_p0, _p1)		catPrim((_p0), (_p1))
#define BrPSPrimTerminate(_p)		termPrim((_p))

#define BrPSPrimSemiTransSet(_p, _abe)	setSemiTrans((_p), (_abe))
#define BrPSPrimShadeTexSet(_p, _tge)	setShadeTex((_p), (_tge))

#define BrPSPrimPolyF3Set(_p)		setPolyF3((_p))
#define BrPSPrimPolyFT3Set(_p)		setPolyFT3((_p))
#define BrPSPrimPolyF4Set(_p)		setPolyF4((_p))
#define BrPSPrimPolyFT4Set(_p)		setPolyFT4((_p))

#define BrPSPrimPolyG3Set(_p)		setPolyG3((_p))
#define BrPSPrimPolyGT3Set(_p)		setPolyGT3((_p))
#define BrPSPrimPolyG4Set(_p)		setPolyG4((_p))
#define BrPSPrimPolyGT4Set(_p)		setPolyGT4((_p))

#define BrPSPrimPolyWFT3Set(_p, _r)								\
			BrPSPrimPolyFT3Set(&(_p)->poly),						\
			BrPSPrimTextureWindowSet(&(_p)->twin, (_r)),				\
			BrPSPrimMerge(&(_p)->twin, &(_p)->poly)

#define BrPSPrimPolyWGT3Set(_p, _r)								\
			BrPSPrimPolyGT3Set(&(_p)->poly),						\
			BrPSPrimTextureWindowSet(&(_p)->twin, (_r)),				\
			BrPSPrimMerge(&(_p)->twin, &(_p)->poly)

#define BrPSPrimPolyMF3Set(_p, _dfe, _dtd, _tpage, _r)						\
			BrPSPrimPolyF3Set(&(_p)->poly),						\
			BrPSPrimDrawModeSet(&(_p)->mode, (_dfe), (_dtd), (_tpage), (_r)),		\
			BrPSPrimMerge(&(_p)->mode, &(_p)->poly)

#define BrPSPrimPolyMG3Set(_p, _dfe, _dtd, _tpage, _r)						\
			BrPSPrimPolyG3Set(&(_p)->poly),						\
			BrPSPrimDrawModeSet(&(_p)->mode, (_dfe), (_dtd), (_tpage), (_r)),		\
			BrPSPrimMerge(&(_p)->mode, &(_p)->poly)

#define BrPSPrimPolyMFT3Set(_p, _dfe, _dtd, _tpage, _r)						\
			BrPSPrimPolyFT3Set(&(_p)->poly),						\
			BrPSPrimDrawModeSet(&(_p)->mode, (_dfe), (_dtd), (_tpage), (_r)),		\
			BrPSPrimMerge(&(_p)->mode, &(_p)->poly)

#define BrPSPrimPolyMGT3Set(_p, _dfe, _dtd, _tpage, _r)						\
			BrPSPrimPolyGT3Set(&(_p)->poly),						\
			BrPSPrimDrawModeSet(&(_p)->mode, (_dfe), (_dtd), (_tpage), (_r)),		\
			BrPSPrimMerge(&(_p)->mode, &(_p)->poly)

#define BrPSPrimSpriteSet(_p)		setSprt((_p))
#define BrPSPrimSprite8Set(_p)		setSprt8((_p))
#define BrPSPrimSprite16Set(_p)		setSprt16((_p))

#define BrPSPrimTileSet(_p)		setTile((_p))
#define BrPSPrimTile1Set(_p)		setTile1((_p))
#define BrPSPrimTile8Set(_p)		setTile8((_p))
#define BrPSPrimTile16Set(_p)		setTile16((_p))

#define BrPSPrimBlockFillSet(_p)		setBlockFill((_p))
#define BrPSPrimDrawMoveSet(_p)		setDrawMove((_p))

#define BrPSPrimLineF2Set(_p)		setLineF2((_p))
#define BrPSPrimLineF3Set(_p)		setLineF3((_p))
#define BrPSPrimLineF4Set(_p)		setLineF4((_p))

#define BrPSPrimLineG2Set(_p)		setLineG2((_p))
#define BrPSPrimLineG3Set(_p)		setLineG3((_p))
#define BrPSPrimLineG4Set(_p)		setLineG4((_p))

#define BrPSPrimTPageSet(_p, _t, _abr, _x, _y)							\
	((p)->tpage = GetTPage((_t), (_abr), (_x), (_y)))

#define BrPSPrimClutSet(_p, _x, _y)								\
	((p)->clut = GetClut((_x), (_y)))

#define BrPSPrimDrawAreaSet(_p, _r)	SetDrawArea((DR_AREA *)(_p), (RECT *)(_r))
#define BrPSPrimDrawEnvironmentSet(_p, _env)							\
					SetDrawEnv((DR_ENV *)(_p), (DRAWENV *)(_env))

#define BrPSPrimDrawModeSet(_p, _disp, _dither, _tp, _tw)						\
					SetDrawMode((DR_MODE *)(_p),				\
						(_disp), (_dither),				\
						(_tp), (RECT *)(_tw))

#define BrPSPrimDrawOffsetSet(_p, _ofs)	SetDrawOffset((DR_OFFSET *)(_p), (_ofs))

#define BrPSPrimPrioritySet(_p, _pbc, _pbw) 							\
					SetPriority((DR_PRIO *)(_p), (_pbc), (_pbw)
#define BrPSPrimTextureWindowSet(_p, _r)	SetTexWindow((DR_TWIN *)(_p), (RECT *)(_r))

#define BrPSDrawEnvironmentDefine(_e, _x, _y, _w, _h)						\
				SetDefDrawEnv((DRAWENV *)(_e), (_x), (_y), (_w), (_h))
#define BrPSDisplayEnvironmentDefine(_e, _x, _y, _w, _h)						\
				SetDefDispEnv((DISPENV *)(_e), (_x), (_y), (_w), (_h))

#define BrPSDrawEnvironmentGet(_p)	GetDrawEnv((DRAWENV *)(_p))
#define BrPSDrawEnvironmentSet(_p)	PutDrawEnv((DRAWENV *)(_p))
#define BrPSDisplayEnvironmentGet(_p)	GetDispEnv((DISPENV *)(_p))
#define BrPSDisplayEnvironmentSet(_p)	PutDispEnv((DISPENV *)(_p))

#define BrPSDrawEnvironmentRGBSet(_p, _r, _g, _b)								\
					(_p)->r = (_r), (_p)->g = (_g), (_p)->b = (_b)

#define BrPSPrimMerge(_p0, _p1)		MargePrim((_p0), (_p1))

#define BrPSDisplayMaskSet(_i)		SetDispMask(_i)
#define BrPSGraphReset(_i)		ResetGraph(_i)
#define BrPSGraphDebugSet(_i)		SetGraphDebug(_i)
#define BrPSGraphDebugGet()		GetGraphDebug()
#define BrPSGraphQueueSet(_m)		SetGraphQueue(_m)
#define BrPSGraphReverseSet(_m)		SetGraphReverse(_m)


/*
 * gte macros
 */
#define BrPSGeomInit()			InitGeom()

#ifdef __cplusplus
};
#endif
#endif
