/*
 * Copyright (c) 1993-1995 by Argonaut Technologies Limited. All rights reserved.
 *
 * $Id: zsproto.h 1.1 1996/09/12 14:02:51 NIKH Exp $
 * $Locker: $
 *
 * Function prototypes for Z buffer renderer
 */
#ifndef _ZSPROTO_H_
#define _ZSPROTO_H_

#ifdef __cplusplus
extern "C" {
#endif

#define BR_BOUNDS_MIN_X	0
#define BR_BOUNDS_MIN_Y	1
#define BR_BOUNDS_MAX_X	2
#define BR_BOUNDS_MAX_Y	3

/*
 * sort type for Z-sort insertion into OT
 */
enum {
	BR_ZSORT_FIXED_CAMERA	=	0x0001,
};

/*
 * Callback function invoked when an actor is
 * rendered
 */
typedef void BR_CALLBACK br_renderbounds_cbfn(
	br_actor *actor,
        const br_model *model,
        const br_material *material,
	br_order_table *order_table,
	br_uint_8	style,
        const br_matrix3t *model_to_screen,
        const br_int_32 bounds[4]);

/*
 * Callback function invoked when primitive is generated
 */
typedef br_uint_32 BR_CALLBACK br_primitive_cbfn(
	void *primitive,
	br_actor *actor,
	br_model *model,
	br_material *material,
	br_order_table *order_table,
	br_uint_16 number,
	br_scalar z[3]);

#ifndef _NO_PROTOTYPES

void BR_PUBLIC_ENTRY BrZsBegin(br_uint_32 *primitive, br_uint_32 primitive_size);
void BR_PUBLIC_ENTRY BrZsEnd(void);

void BR_PUBLIC_ENTRY BrZsSceneRenderBegin(br_actor *world,
					br_actor *camera,
                                        br_pixelmap *viewport);

void BR_PUBLIC_ENTRY BrZsSceneRenderAdd(br_actor *tree);
void BR_PUBLIC_ENTRY BrZsSceneRenderEnd(void);

/*
 * Wrapper that inokes above therr calls in order
 */
void BR_PUBLIC_ENTRY BrZsSceneRender(br_actor *world,
					br_actor *camera,
                                        br_pixelmap *viewport);

/*
 * Used within custom model callbacks to render other models
 */
void BR_PUBLIC_ENTRY BrZsModelRender(br_actor *actor,
				  br_model *model,
                                  br_material *material,
				  br_order_table *order_table,
				  br_uint_8 style,
				  int on_screen,
				  int use_custom);

br_renderbounds_cbfn * BR_PUBLIC_ENTRY BrZsRenderBoundsCallbackSet(br_renderbounds_cbfn *new_cbfn);
br_primitive_cbfn * BR_PUBLIC_ENTRY BrZsPrimitiveCallbackSet(br_primitive_cbfn *new_cbfn);

br_order_table *BR_PUBLIC_ENTRY BrZsOrderTableAllocate(br_uint_16 size, br_uint_32 flags, br_uint_16 type);
void BR_PUBLIC_ENTRY BrZsOrderTableFree(br_order_table *order_table);

br_order_table *BR_PUBLIC_ENTRY BrZsActorOrderTableSet(br_actor *actor, br_order_table *order_table);
br_order_table *BR_PUBLIC_ENTRY BrZsActorOrderTableGet(const br_actor *actor);

br_order_table *BR_PUBLIC_ENTRY BrZsDefaultOrderTableGet(void);
void BR_PUBLIC_ENTRY BrZsDefaultOrderTableClear(void);

br_order_table *BR_PUBLIC_ENTRY BrZsCurrentOrderTableGet(void);

void BR_PUBLIC_ENTRY BrZsOrderTableClear(br_order_table *order_table);
void BR_PUBLIC_ENTRY BrZsOrderTablePrimitiveInsert(br_order_table *order_table, void *primitive, br_uint_16 bucket);

br_vector2 * BR_PUBLIC_ENTRY BrZsSceneFogSet(br_uint_32 colour, br_vector2 *bounds);

br_uint_32 BR_PUBLIC_ENTRY BrZsPrimitiveBucketSelect(const br_scalar *z, br_uint_16 type, br_scalar min_z, br_scalar max_z, br_uint_16 size, br_uint_16 sort_type);

void BR_PUBLIC_ENTRY BrZsOrderTablePrimaryDisable(void);
void BR_PUBLIC_ENTRY BrZsOrderTablePrimaryEnable(br_order_table *order_table);

#ifdef __cplusplus
};
#endif
#endif
#endif
