/* Copyright (c) 1993-1996 Argonaut Technologies Limited. All rights reserved.
 *
 * $Id: HARNESS.C 1.1 1996/09/12 14:01:09 NIKH Exp $
 * $Locker: $
 *
 * Generic test harness framework
 */

/*
 * BRender headers
 */
#include "brender.h"
#include "psio.h"

/*
 * Generic header
 */
#include "harness.h"
#include "support.h"

/*
 * main screen (device)
 */
br_pixelmap *screen;

/*
 * double buffered back screen
 */
br_pixelmap *back_screen[2];

/*
 * sample harness function table
 */
_sample_fn_list *_sample_fn_table;
br_uint_8 debug = 1;

br_uint_32 *bucket[2];

void main(void) {

	br_int_32 frame_mode;	/* frame mode for VSync */
	br_uint_32 time, max_time;
	
	int count;
	max_time = 0;
	time = 0;

	/*
	 * init PS hardware, BRender, PSGfx
	 * setup NTSC/PAL, GPU
	 */
	frame_mode = _sample_fn_table->sample_init_device();

	/*
	 * setup data structures
	 */
	_sample_fn_table->sample_init_world();

	time = VSync(frame_mode);
	/*
	 * inner loop, terminated by sample_loop_test() returning -1
	 */
	while(_sample_fn_table->sample_loop_test() != -1) {

	    	/*
		 * generic pad test
		 */
		_sample_fn_table->sample_pad_test();

		/*
		 * update data structures
		 */
		_sample_fn_table->sample_update_fn();

		/*
		 * render world
		 */
		_sample_fn_table->sample_render_fn();

		count = 1; /* root */

		BrActorEnum(root, ActorEnumCbfn, &count);

		if(debug == 1)
			FntPrint("%3d Actors\n%3d Hz\n%3d hbl", count, VBL_PER_SEC / ((time / HBL_PER_VBL) + 1), time, max_time);
		/*
		 * align double buffer swap
		 */
		{
			br_uint_8 db_index = PSGfxDoubleBufferIndexRead();

			DrawSync(0);

			time = VSync(frame_mode);
			if(time > max_time) max_time = time;
		}

		if(debug == 1)
			FntFlush(-1);
		_sample_fn_table->sample_double_buffer();
	}

	/*
	 * terminate sample
	 */
	_sample_fn_table->sample_term();
}

