/* Copyright (c) 1993-1996 Argonaut Technologies Limited. All rights reserved.
 *
 * $Id: SAMPLE0.C 1.1 1996/09/12 14:01:14 NIKH Exp $
 * $Locker: $
 *
 * PS Generic test harness sample
 *
 * Sample0 - minimal demo (spinning cube)
 * default model, order table inherited from root, one light
 */

/*
 * Sony library headers
 */
#include <sys/types.h>
#include <r3000.h>
#include <asm.h>
#include <kernel.h>
#include <libgte.h>
#include <libgpu.h>
#include <libetc.h>

/*
 * BRender headers
 */
#include "brender.h"
#include "psio.h"

/*
 * test harness header
 */
#include "harness.h"

/*
 * test harness generic functions for samples
 */
#include "support.h"

/*
 * sample0 prototypes
 */
br_int_8 Sample0_InitWorld(void);
br_int_8 Sample0_RenderFn(void);
br_int_8 Sample0_UpdateFn(void);
br_int_8 Sample0_PadTest(void);


_sample_fn_list sample0_fn_table = {

	_GenericInitDevice,			/* init PS hardware, BRender & PSGfx */
	Sample0_InitWorld,			/* init data structures */
	Sample0_RenderFn,			/* rendering function */
	_GenericDoubleBuffer,			/* double buffer function */
	Sample0_UpdateFn,			/* update world */
	NULL,					/* double buffer callback */
	_GenericLoopTest,			/* loop decision */
	Sample0_PadTest,			/* joypad */
	_GenericTerminate,			/* terminate sample (shut down BRender, PSGfx) */
	
};

/*
 * setup test harness control table
 */
_sample_fn_list *_sample_fn_table = &sample0_fn_table;

/*
 * Sample0 data
 */
br_scalar xpos = 0, ypos = 0, zpos = BR_SCALAR(-2);
br_matrix3t rotation;
br_actor *actor;


int counter = 0, animate = 1;
 
/*
 * setup data structures
 */
br_int_8 Sample0_InitWorld(void)
{
	BrMatrix3tIdentity(&rotation);

	/*
	 * allocate BR_ACTOR_MODEL which uses default model, inherited root order table
	 */
	actor = BrActorAdd(root, BrActorAllocate(BR_ACTOR_MODEL, NULL));
	BrMatrix3tTranslate(&actor->t.t.mat3t.mat, xpos, ypos, zpos);
}

/*
 * BRender Rendering function
 */
br_int_8 Sample0_RenderFn(void)
{
	br_uint_8 db_index = PSGfxDoubleBufferIndexRead();
    
        BrZsSceneRender(root, camera, back_screen[db_index]);
}

/*
 * data Update function
 */
br_int_8 Sample0_UpdateFn(void)
{
	counter = (counter >= 360)?counter + 2 - 360:counter + 2;
		
	BrMatrix3tTranslate(&actor->t.t.mat3t.mat ,xpos, ypos, zpos);

	BrMatrix3tPre(&actor->t.t.mat3t.mat, &rotation);

	if(animate)
		BrMatrix3tPreRotateY(&actor->t.t.mat3t.mat, BR_ANGLE_DEG(counter));
}

/*
 * Pad test
 */
br_int_8 Sample0_PadTest(void)
{
	int ret = 0;

	unsigned long  padd = PadRead(1);

	if(PAD_COMPARE(padd, PAD_UP)) ypos = BR_ADD(ypos,BR_SCALAR(0.05));
	if(PAD_COMPARE(padd, PAD_DOWN)) ypos = BR_SUB(ypos,BR_SCALAR(0.05));

	if(PAD_COMPARE(padd, PAD_LEFT)) xpos = BR_SUB(xpos,BR_SCALAR(0.05));
	if(PAD_COMPARE(padd, PAD_RIGHT)) xpos = BR_ADD(xpos,BR_SCALAR(0.05));

	if(PAD_COMPARE(padd, PAD_TRI)) BrMatrix3tPreRotateX(&rotation,BR_ANGLE_DEG(2.0));
	if(PAD_COMPARE(padd, PAD_CROSS)) BrMatrix3tPreRotateX(&rotation,BR_ANGLE_DEG(-2.0));

	if(PAD_COMPARE(padd, PAD_FRONT_LTOP)) {

	    	if(PAD_COMPARE(padd, PAD_SQUARE)) BrMatrix3tPreRotateZ(&rotation, BR_ANGLE_DEG(2.0));
		if(PAD_COMPARE(padd, PAD_CIRCLE)) BrMatrix3tPreRotateZ(&rotation, BR_ANGLE_DEG(-2.0));
	} else {

		if(PAD_COMPARE(padd, PAD_SQUARE)) BrMatrix3tPreRotateY(&rotation,BR_ANGLE_DEG(-2.0));
		if(PAD_COMPARE(padd, PAD_CIRCLE)) BrMatrix3tPreRotateY(&rotation,BR_ANGLE_DEG(2.0));
	}

	if(PAD_COMPARE(padd, PAD_FRONT_RTOP)) {
	    	zpos = BR_ADD(zpos,BR_SCALAR(0.5));
	}

	if(PAD_COMPARE(padd, PAD_FRONT_RBOT)) {
		zpos = BR_SUB(zpos,BR_SCALAR(0.5));
	}

	if(PAD_COMPARE(padd, PAD_FRONT_LBOT)) {
	    	animate ^= 1;

		while(PAD_COMPARE(PadRead(1), PAD_FRONT_LBOT));
	}

	return ret;
}
