/* Copyright (c) 1993-1996 Argonaut Technologies Limited. All rights reserved.
 *
 * $Id: SAMPLE10.C 1.1 1996/09/12 14:01:19 NIKH Exp $
 * $Locker: $
 *
 * PS Generic test harness sample
 *
 * Sample9 - textured model (clone demo)
 * model 'test.dat', pixelmap 'stars.pix', multiple order tables, one light
 */

/*
 * Sony library headers
 */
#include <sys/types.h>
#include <r3000.h>
#include <asm.h>
#include <kernel.h>
#include <libgte.h>
#include <libgpu.h>
#include <libetc.h>

/*
 * BRender headers
 */
#include "brender.h"
#include "psio.h"

/*
 * test harness header
 */
#include "harness.h"

/*
 * test harness generic functions for samples
 */
#include "support.h"

/*
 * Sample9 prototypes
 */
br_int_8 Sample9_InitWorld(void);
br_int_8 Sample9_RenderFn(void);
br_int_8 Sample9_UpdateFn(void);
br_int_8 Sample9_PadTest(void);

#define BrPSPrimPolyMFT4Set(_p, _dfe, _dtd, _tpage, _r)						\
			BrPSPrimPolyFT4Set(&(_p)->poly),					\
			BrPSPrimDrawModeSet(&(_p)->mode, (_dfe), (_dtd), (_tpage), (_r)),	\
			BrPSPrimMerge(&(_p)->mode, &(_p)->poly)

_sample_fn_list Sample9_fn_table = {

	_GenericInitDevice,			/* init PS hardware, BRender & PSGfx */
	Sample9_InitWorld,			/* init data structures */
	Sample9_RenderFn,			/* rendering function */
	_GenericDoubleBuffer,			/* double buffer function */
	Sample9_UpdateFn,			/* update world */
	NULL,					/* double buffer callback */
	_GenericLoopTest,			/* loop decision */
	Sample9_PadTest,			/* joypad */
	_GenericTerminate,			/* terminate sample (shut down BRender, PSGfx) */
	
};

/*
 * setup test harness control table
 */
_sample_fn_list *_sample_fn_table = &Sample9_fn_table;

int counter = 0, animate = 1, counter1, counter2, counter3, auto_change, frames;
br_scalar xpos, ypos, zpos;
extern br_uint_8 debug;

struct {
    
	char *filename;
	br_uint_16 clut_x;
	br_uint_16 clut_y;
	br_uint_8 palette_trans;
	br_uint_16 tpage_x;
	br_uint_16 tpage_y;
	    
} Bitmaps[] = {

	{"spship.pix", 512, 256+241, 1, 448, 0},
	{"logo.pix", 512, 256+242, 0, 320, 0},
	{"floor_0.pix", 512, 256+240, 1, 576, 0},
	{"plant.pix", 512, 256+244, 0, 640, 0},
	{"brender.pix", 512, 256+243, 0, 320, 256},
	{"cloud.pix", 512, 256+245, 0, 448, 256},
	{"column.pix", 512, 256+246, 1, 512, 256},
    
};

br_actor *pivot0, *floor, *object, *column[6], *pivot1;
br_order_table *primary_order_table;
br_matrix3t *loop;

typedef struct brps_prim_poly_mft4 {

	void *p;
	brps_prim_draw_mode mode;
	brps_prim_poly_ft4 poly;
    
} brps_prim_poly_mft4;

brps_prim_poly_mft4 logo[2];
brps_prim_poly_mft4 brender[2];

br_uint_32 *ot_start[2];
br_uint_32 CameraLookAt;

typedef void camera_fn(void);

void CameraReset0(void)
{
	BrMatrix3tIdentity(&pivot1->t.t.mat3t.mat);
	camera->t.type = BR_TRANSFORM_LOOK_UP;
	xpos = 0; ypos = BR_SCALAR(-6); zpos = BR_SCALAR(-30);
	CameraLookAt=1;
}
void CameraUpdate0(void)
{
}

void CameraReset1(void)
{
	camera->t.type = BR_TRANSFORM_LOOK_UP;
	xpos = 0; ypos = BR_SCALAR(-7); zpos = BR_SCALAR(8);
	CameraLookAt=1;
}
void CameraUpdate1(void)
{
}
void CameraReset2(void)
{
	camera->t.type = BR_TRANSFORM_MATRIX3T;
	CameraLookAt=0;
}
void CameraUpdate2(void)
{
	BrMatrix3tCopy(&camera->t.t.mat3t.mat, loop + ((counter + 170)%180));
	BrMatrix3tPreTranslate(&camera->t.t.mat3t.mat, xpos, ypos, zpos);
}
void CameraReset3(void)
{
	BrMatrix3tIdentity(&pivot1->t.t.mat3t.mat);
	camera->t.type = BR_TRANSFORM_LOOK_UP;
	xpos = 0; ypos = BR_SCALAR(-20); zpos = BR_SCALAR(-50);
	CameraLookAt=1;
}
void CameraUpdate3(void)
{
	BrMatrix3tRotateY(&pivot1->t.t.mat3t.mat, BR_ANGLE_DEG(counter1));
}

struct {

	camera_fn *update;
	camera_fn *reset;
	br_uint_32 frames;

} CameraModes[] = {

	{	CameraUpdate0, CameraReset0, 1800,	},
	{	CameraUpdate1, CameraReset1, 1800,	},
	{	CameraUpdate2, CameraReset2, 900,	},
	{	CameraUpdate3, CameraReset3, 1800,	},
};

br_uint_32 NumCameraModes, CurrentCameraMode;

brps_prim_poly_ft4 start[2];

void VBL_cbfn(br_uint_8 db_index)
{
	br_uint_32 *db_table;
	br_order_table *ot;
	
        BrZsSceneRender(root, camera, back_screen[db_index]);

	ot = BrZsActorOrderTableGet(root);
	    
	db_table = ((br_uint_32 *)ot->table[db_index]) + 2;
	BrPSPrimAdd(db_table, &start[db_index]);

	BrPixelmapDoubleBuffer(screen, back_screen[db_index]);
}

/*
 * setup data structures
 */
br_int_8 Sample9_InitWorld(void)
{
	br_pixelmap *pm, *texture;
	br_model *model;
	br_material *mats[32];
	br_order_table *order_table;
	
	int i;

	/*
	 * sneak some VBL interrupt code in here while loading data...
	 */
	{
		brps_rectangle rect;
                if((texture = BrPixelmapLoad("pssml.pix")) == NULL)
                        BR_ERROR0("Unable to load 'pssml.pix' from host");
			    
		BrPSRectangleSet(&rect, 576, 256, texture->width/2, texture->height);
		LoadImage((RECT *)&rect, (unsigned long *)texture->pixels);


		BrPSRectangleSet(&rect, 576, 425, 256, 1);
		LoadImage((RECT *)&rect, (unsigned long *)texture->map->pixels);

		BrPSPrimColour0Set(&start[0], 100, 100, 100);
		BrPSPrimColour0Set(&start[1], 100, 100, 100);

		BrPSPrimVertex0Set(&start[0], (SCREEN_WIDTH - texture->width)/2, (SCREEN_HEIGHT - texture->height)/2);
		BrPSPrimVertex1Set(&start[0], ((SCREEN_WIDTH - texture->width)/2) + texture->width, (SCREEN_HEIGHT - texture->height)/2);
		BrPSPrimVertex2Set(&start[0], (SCREEN_WIDTH - texture->width)/2, ((SCREEN_HEIGHT - texture->height)/2) + texture->height);
		BrPSPrimVertex3Set(&start[0], ((SCREEN_WIDTH - texture->width)/2) + texture->width, ((SCREEN_HEIGHT - texture->height)/2) + texture->height);

		BrPSPrimVertex0Set(&start[1], (SCREEN_WIDTH - texture->width)/2, (SCREEN_HEIGHT - texture->height)/2);
		BrPSPrimVertex1Set(&start[1], ((SCREEN_WIDTH - texture->width)/2) + texture->width, (SCREEN_HEIGHT - texture->height)/2);
		BrPSPrimVertex2Set(&start[1], (SCREEN_WIDTH - texture->width)/2, ((SCREEN_HEIGHT - texture->height)/2) + texture->height);
		BrPSPrimVertex3Set(&start[1], ((SCREEN_WIDTH - texture->width)/2) + texture->width, ((SCREEN_HEIGHT - texture->height)/2) + texture->height);

		BrPixelmapFree(texture);

		setTPage(&start[0], 1, 0, 576, 256);
		setTPage(&start[1], 1, 0, 576, 256);

		setClut(&start[0], 576, 425);
		setClut(&start[1], 576, 425);

		BrPSPrimMap4Set(&start[0], 0, 0, 168, 0, 0, 168, 168, 168);
		BrPSPrimMap4Set(&start[1], 0, 0, 168, 0, 0, 168, 168, 168);

		BrPSPrimPolyFT4Set(&start[0]);
		BrPSPrimPolyFT4Set(&start[1]);

		PSGfxVSyncCallbackSet(VBL_cbfn);
	}

	NumCameraModes = BR_ASIZE(CameraModes);
	CurrentCameraMode = 0;
	frames = CameraModes[CurrentCameraMode].frames;
	auto_change = 1;

	CameraModes[CurrentCameraMode].reset();

	for(i=0; i<BR_ASIZE(Bitmaps); i++) {

		if((pm = BrPixelmapLoad(Bitmaps[i].filename)) == NULL)
			BR_ERROR1("Unable to load '%s' from host", Bitmaps[i].filename);

		/*
		 * copy pixelmap to VRAM
		 */
		texture = BrPixelmapAllocateSub(screen, Bitmaps[i].tpage_x, Bitmaps[i].tpage_y, pm->width, pm->height);

		texture->identifier = BrMemStrDup(pm->identifier);
                BrPixelmapRectangleCopy(texture, -texture->origin_x, -texture->origin_y, pm, -pm->origin_x, -pm->origin_y, pm->width, pm->height);
		BrPixelmapTPageSet(texture, 1);

		BrPixelmapClutSet(texture, Bitmaps[i].clut_x, Bitmaps[i].clut_y);
                BrPixelmapRectangleCopy(screen, Bitmaps[i].clut_x-screen->origin_x, Bitmaps[i].clut_y-screen->origin_y, pm->map, -pm->map->origin_x, -pm->map->origin_y, pm->map->width, pm->map->height);
	
		BrPixelmapFree(pm);
		BrMapAdd(texture);
	}

	/*
	 * load some materials from script
	 */
	i = BrFmtScriptMaterialLoadMany("material.scr", mats, 32);
	BrMaterialAddMany(mats, i);

	/*
	 * load model
	 */
	pivot0 = BrActorAllocate(BR_ACTOR_NONE, NULL);
	pivot0->t.type = BR_TRANSFORM_MATRIX3T;
	BrMatrix3tTranslate(&pivot0->t.t.mat3t.mat, xpos, ypos, zpos);

	pivot1 = BrActorAdd(pivot0, BrActorAllocate(BR_ACTOR_NONE, NULL));
	pivot1->t.type = BR_TRANSFORM_MATRIX3T;
	BrMatrix3tIdentity(&pivot1->t.t.mat3t.mat);
	 

	floor = BrActorAdd(pivot1, BrActorAllocate(BR_ACTOR_MODEL, NULL));
	model = BrModelLoad("floor.dat");
	
	for(i=0; i<model->nvertices; i++) {
		model->vertices[i].red = 128;
		model->vertices[i].grn = 128;
		model->vertices[i].blu = 128;
	}

	floor->model = BrModelAdd(model);
	floor->t.type = BR_TRANSFORM_MATRIX3T;
	BrMatrix3tTranslate(&floor->t.t.mat3t.mat, 0, BR_SCALAR(-5), 0);

	model = BrModelLoad("column.dat");
	for(i=0; i<model->nvertices; i++) {
		model->vertices[i].red = 128;
		model->vertices[i].grn = 128;
		model->vertices[i].blu = 128;
	}
	BrModelAdd(model);

	for(i=0; i<4; i++) {
		column[i] = BrActorAdd(pivot1, BrActorAllocate(BR_ACTOR_MODEL, NULL));
		column[i]->model = BrModelAdd(BrModelClone(model));
		column[i]->t.type = BR_TRANSFORM_MATRIX3T;

		BrMatrix3tTranslate(&column[i]->t.t.mat3t.mat, BR_SCALAR(18), 0, 0);
		BrMatrix3tPostRotateY(&column[i]->t.t.mat3t.mat, BR_ANGLE_DEG(i*(360/4) + (360/4)/2));
	}

	object = BrActorAdd(pivot1, BrActorAllocate(BR_ACTOR_MODEL, NULL));
	object->model = BrModelAdd(BrModelLoad("ship.dat"));
	object->t.type = BR_TRANSFORM_MATRIX3T;

        primary_order_table = BrZsOrderTableAllocate(2048, BR_ORDER_TABLE_LEAVE_BOUNDS, BR_SORT_MIN);
        primary_order_table->min_z = 0;
        primary_order_table->max_z = 20000;

        BrZsActorOrderTableSet(object, BrZsOrderTableAllocate(1024, BR_ORDER_TABLE_NEW_BOUNDS | BR_ORDER_TABLE_SORT_NEAR, BR_SORT_MAX));

	/*
	 * prepare logo
	 */
	pm = BrMapFind("logo.pix");
	{
		brps_rectangle rect;

		BrPSRectangleSet(&rect, 0, 0, 255, 255);
		BrPSPrimPolyMFT4Set(&logo[0], 0, 0, pm->tpage, &rect);
		BrPSPrimPolyMFT4Set(&logo[1], 0, 0, pm->tpage, &rect);

		logo[0].poly.tpage = pm->tpage;
		logo[1].poly.tpage = pm->tpage;

		BrPSPrimMap4Set(&logo[0].poly, 0, 0, 31, 0, 0, 31, 31, 31);
		BrPSPrimMap4Set(&logo[1].poly, 0, 0, 31, 0, 0, 31, 31, 31);

		logo[0].poly.clut = pm->clut;
		logo[1].poly.clut = pm->clut;

		BrPSPrimVertex0Set(&logo[0].poly, 304-64, 16);
		BrPSPrimVertex1Set(&logo[0].poly, 304, 16);
		BrPSPrimVertex2Set(&logo[0].poly, 304-64, 80);
		BrPSPrimVertex3Set(&logo[0].poly, 304, 80);

		BrPSPrimVertex0Set(&logo[1].poly, 304-64, 16);
		BrPSPrimVertex1Set(&logo[1].poly, 304, 16);
		BrPSPrimVertex2Set(&logo[1].poly, 304-64, 80);
		BrPSPrimVertex3Set(&logo[1].poly, 304, 80);

		BrPSPrimColour0Set(&logo[0].poly, 128, 128, 128);
		BrPSPrimColour0Set(&logo[1].poly, 128, 128, 128);

		BrPSPrimTerminate(&logo[0].mode);
		BrPSPrimTerminate(&logo[1].mode);
	}
	
	/*
	 * prepare brender
	 */
	pm = BrMapFind("brender.pix");
	{
		brps_rectangle rect;

		BrPSRectangleSet(&rect, 0, 0, 255, 255);
		BrPSPrimPolyMFT4Set(&brender[0], 0, 0, pm->tpage, &rect);
		BrPSPrimPolyMFT4Set(&brender[1], 0, 0, pm->tpage, &rect);

		brender[0].poly.tpage = pm->tpage;
		brender[1].poly.tpage = pm->tpage;

		BrPSPrimMap4Set(&brender[0].poly, 0, 0, 63, 0, 0, 63, 63, 63);
		BrPSPrimMap4Set(&brender[1].poly, 0, 0, 63, 0, 0, 63, 63, 63);

		brender[0].poly.clut = pm->clut;
		brender[1].poly.clut = pm->clut;

		BrPSPrimVertex0Set(&brender[0].poly, 16, 16);
		BrPSPrimVertex1Set(&brender[0].poly, 80, 16);
		BrPSPrimVertex2Set(&brender[0].poly, 16, 80);
		BrPSPrimVertex3Set(&brender[0].poly, 80, 80);

		BrPSPrimVertex0Set(&brender[1].poly, 16, 16);
		BrPSPrimVertex1Set(&brender[1].poly, 80, 16);
		BrPSPrimVertex2Set(&brender[1].poly, 16, 80);
		BrPSPrimVertex3Set(&brender[1].poly, 80, 80);

		BrPSPrimColour0Set(&brender[0].poly, 128, 128, 128);
		BrPSPrimColour0Set(&brender[1].poly, 128, 128, 128);

		BrPSPrimAdd(&logo[0].mode, &brender[0].mode);
		BrPSPrimAdd(&logo[1].mode, &brender[1].mode);
	}

	ot_start[0] = ((br_uint_32 *)primary_order_table->table[0]);
	ot_start[1] = ((br_uint_32 *)primary_order_table->table[1]);

	/*
	 * reset VBL callback
	 */
	PSGfxVSyncCallbackSet(NULL);

	BrZsOrderTablePrimaryEnable(primary_order_table);
	BrZsActorOrderTableSet(root, primary_order_table);

	BrActorAdd(root, pivot0);

	counter = 0, counter1=0, counter2 = 0, counter3 = 0;

	loop = BrMemAllocate(sizeof(br_matrix3t) * 360, BR_MEMORY_APPLICATION);

	for(i=0; i<=180; i++) {

		counter += 2;
		if(counter >= 360) counter -= 360;
		counter1 += 4;
		if(counter1 >= 360) counter1 -= 360;

		BrMatrix3tTranslate(&object->t.t.mat3t.mat, BR_ADD(BR_MUL(BR_SIN(BR_ANGLE_DEG(counter1)), BR_SCALAR(8)), BR_SCALAR(18)), BR_SIN(BR_ANGLE_DEG(counter1*2)), 0);
		BrMatrix3tPostRotateY(&object->t.t.mat3t.mat, BR_ANGLE_DEG(counter));
		BrMatrix3tPreRotateZ(&object->t.t.mat3t.mat, BR_CONST_DIV(BR_SIN(BR_ANGLE_DEG(counter1)), 10));

		BrMatrix3tCopy(loop + i, &object->t.t.mat3t.mat);
	}
	counter=0, counter1=0;
}

/*
 * BRender Rendering function
 */
br_int_8 Sample9_RenderFn(void)
{
	br_uint_8 db_index = PSGfxDoubleBufferIndexRead();

        BrZsSceneRenderBegin(root, camera, back_screen[db_index]);

	BrZsSceneRenderAdd(pivot0);

	BrZsSceneRenderEnd();

	{
		br_uint_32 x,y;

		x = (counter2%8)*32;
		y = (counter2/8)*32;

		BrPSPrimMap4Set(&logo[db_index].poly, x, y, x + 31, y, x, y + 31, x + 31, y + 31);

		x = (counter3%4)*64;
		y = (counter3/4)*64;

		BrPSPrimMap4Set(&brender[db_index].poly, x, y, x + 63, y, x, y + 63, x + 63, y + 63);

		DrawOTag((br_uint_32 *)&logo[db_index].mode);
	}
}

/*
 * data Update function
 */
br_int_8 Sample9_UpdateFn(void)
{
	br_transform xform;

	counter++;
	if(counter >= 180) counter -= 180;

	counter1 += 2;
	if(counter1 >= 360) counter1 -= 360;
	
	counter2++;
	if(counter2 >= 60) counter2 -= 60;

	if((counter2 % 2) == 0)
		counter3 += 1;
	if(counter3 == 64) counter3 = 0;

	if(animate == 1) {
		BrMatrix3tCopy(&object->t.t.mat3t.mat, loop + counter);
	}

	if(CameraLookAt == 1) {
		BrVector3SetInt(&camera->t.t.look_up.up, 0, 1, 0);
		BrVector3Set(&camera->t.t.look_up.t, 0, 0, 0);
	}

	BrMatrix3tTranslate(&pivot0->t.t.mat3t.mat, xpos, ypos, zpos);

	CameraModes[CurrentCameraMode].update();

	if(CameraLookAt == 1) {
		BrActorToActorMatrix34(&xform.t.mat, object, camera->parent);
		BrVector3Sub(&camera->t.t.look_up.look, &xform.t.translate.t, &camera->t.t.look_up.t);
	}
}

/*
 * Pad test
 */
br_int_8 Sample9_PadTest(void)
{
	int ret = 0;

	unsigned long  padd = PadRead(1);
	
	if(auto_change == 1) {

		frames--;
		if(frames == 0)
			padd |= PAD_START;
	}

	if(PAD_COMPARE(padd, PAD_FRONT_LTOP)) {

		debug ^= 1;    

		while(PAD_COMPARE(PadRead(1), PAD_FRONT_LTOP));
	}
	
	if(PAD_COMPARE(padd, PAD_FRONT_LBOT)) {
	    	animate ^= 1;

		while(PAD_COMPARE(PadRead(1), PAD_FRONT_LBOT));
	}

	if(PAD_COMPARE(padd, PAD_CROSS)) {

		auto_change ^= 1;

		while(PAD_COMPARE(PadRead(1), PAD_CROSS));
	}

	if(PAD_COMPARE(padd, PAD_START)) {

		CurrentCameraMode = (CurrentCameraMode + 1) % NumCameraModes;
		CameraModes[CurrentCameraMode].reset();

		frames = CameraModes[CurrentCameraMode].frames;

		while(PAD_COMPARE(PadRead(1), PAD_START));
	}

	if(PAD_COMPARE(padd, PAD_UP)) ypos = BR_ADD(ypos,BR_SCALAR(0.5));
	if(PAD_COMPARE(padd, PAD_DOWN)) ypos = BR_SUB(ypos,BR_SCALAR(0.5));

	if(PAD_COMPARE(padd, PAD_LEFT)) xpos = BR_SUB(xpos,BR_SCALAR(0.5));
	if(PAD_COMPARE(padd, PAD_RIGHT)) xpos = BR_ADD(xpos,BR_SCALAR(0.5));

	if(PAD_COMPARE(padd, PAD_FRONT_RTOP)) {
	    	zpos = BR_ADD(zpos,BR_SCALAR(0.5));
	}

	if(PAD_COMPARE(padd, PAD_FRONT_RBOT)) {
		zpos = BR_SUB(zpos,BR_SCALAR(0.5));
	}

	if(ypos > BR_SCALAR(3)) ypos = BR_SCALAR(3);
	if(ypos < BR_SCALAR(-60)) ypos = BR_SCALAR(-60);

	if(zpos > BR_SCALAR(80)) zpos = BR_SCALAR(80);
	if(zpos < BR_SCALAR(-80)) zpos = BR_SCALAR(-80);

	if(xpos > BR_SCALAR(80)) xpos = BR_SCALAR(80);
	if(xpos < BR_SCALAR(-80)) xpos = BR_SCALAR(-80);

	if(PAD_COMPARE(padd, PAD_SELECT)) {

		br_pixelmap *pm;
		br_uint_8 db_index = PSGfxDoubleBufferIndexRead();

		/*
		 * screen capture - useful for checking VRAM
		 */
		pm = BrPixelmapAllocate(back_screen[db_index]->type,
					back_screen[db_index]->width,
					back_screen[db_index]->height,
					NULL,
					0);

		BrPixelmapCopy(pm, back_screen[db_index]);

		DrawSync(0);

		BrPixelmapSave("out.pix", pm);

		BrPixelmapFree(pm);

		while(PAD_COMPARE(PadRead(1), PAD_SELECT));	    
	}

	
	if(debug == 1) {

		FntPrint("\n\n\n\n\n\n\n");
		FntPrint("Camera %d\n", CurrentCameraMode);
		FntPrint("x %d\ny %d\nz %d\n", BrScalarToInt(xpos), BrScalarToInt(ypos), BrScalarToInt(zpos));

		if(auto_change == 1) {
			FntPrint("%3d Auto\n", frames/60);
		}
	}

	return ret;
}
