/* Copyright (c) 1993-1996 Argonaut Technologies Limited. All rights reserved.
 *
 * $Id: SAMPLE12.C 1.1 1996/09/12 14:01:20 NIKH Exp $
 * $Locker: $
 *
 * PS Generic test harness sample
 *
 * Sample12 - textured model
 * model 'test.dat', pixelmap 'test.pix', order table inherited from root, one light
 */

/*
 * Sony library headers
 */
#include <sys/types.h>
#include <r3000.h>
#include <asm.h>
#include <kernel.h>
#include <libgte.h>
#include <libgpu.h>
#include <libetc.h>

/*
 * BRender headers
 */
#include "brender.h"
#include "psio.h"

/*
 * test harness header
 */
#include "harness.h"

/*
 * test harness generic functions for samples
 */
#include "support.h"

/*
 * Sample12 prototypes
 */
br_int_8 Sample12_InitWorld(void);
br_int_8 Sample12_RenderFn(void);
br_int_8 Sample12_UpdateFn(void);
br_int_8 Sample12_PadTest(void);


_sample_fn_list Sample12_fn_table = {

	_GenericInitDevice,			/* init PS hardware, BRender & PSGfx */
	Sample12_InitWorld,			/* init data structures */
	Sample12_RenderFn,			/* rendering function */
	_GenericDoubleBuffer,			/* double buffer function */
	Sample12_UpdateFn,			/* update world */
	NULL,					/* double buffer callback */
	_GenericLoopTest,			/* loop decision */
	Sample12_PadTest,			/* joypad */
	_GenericTerminate,			/* terminate sample (shut down BRender, PSGfx) */
	
};

/*
 * setup test harness control table
 */
_sample_fn_list *_sample_fn_table = &Sample12_fn_table;

/*
 * Sample12 data
 */
br_scalar xpos = 0, ypos = 0, zpos = BR_SCALAR(-1);
br_matrix3t rotation;
br_actor *actor;


int counter = 0, animate = 1;

brps_prim_poly_ft4 start[2];

void VBL_cbfn(br_uint_8 db_index)
{
	br_uint_32 *db_table;
	br_order_table *ot;
	
        BrZsSceneRender(root, camera, back_screen[db_index]);

	ot = BrZsActorOrderTableGet(root);
	    
	db_table = ((br_uint_32 *)ot->table[db_index]) + 1;
	BrPSPrimAdd(db_table, &start[db_index]);

	BrPixelmapDoubleBuffer(screen, back_screen[db_index]);
}

typedef struct br_pick_nearest {
	br_actor *actor;
	br_model *model;
	br_material *material;
	br_vector3 point;
	int face;
	int edge;
	int vertex;
	br_vector2 map;

	br_scalar t;
	br_actor *temp_actor;
} br_pick_nearest;

static br_pick_nearest pickNearest;

/*
 * Global pointer for picking material
 */
br_material *PickMaterial[2];

#define BrPSPrimPolyMFT4Set(_p, _dfe, _dtd, _tpage, _r)						\
			BrPSPrimPolyFT4Set(&(_p)->poly),						\
			BrPSPrimDrawModeSet(&(_p)->mode, (_dfe), (_dtd), (_tpage), (_r)),		\
			BrPSPrimMerge(&(_p)->mode, &(_p)->poly)
typedef struct brps_prim_poly_mft4 {

	void *p;
	brps_prim_draw_mode mode;
	brps_prim_poly_ft4 poly;
    
} brps_prim_poly_mft4;

brps_prim_poly_mft4 cursor[2];
br_int_32 cursor_x = 160, cursor_y = 128;


/*
 * setup data structures
 */
br_int_8 Sample12_InitWorld(void)
{
	br_pixelmap *pm, *texture;
	br_model *model;
	br_material *mats[32];
	br_material *material;
	
	int i, tpage_type;

	/*
	 * sneak some VBL interrupt code in here while loading data...
	 */
	{
		brps_rectangle rect;

		texture = BrPixelmapLoad("pssml.pix");
			    
		BrPSRectangleSet(&rect, 512, 256, texture->width/2, texture->height);
		LoadImage((RECT *)&rect, (unsigned long *)texture->pixels);


		BrPSRectangleSet(&rect, 512, 425, 256, 1);
		LoadImage((RECT *)&rect, (unsigned long *)texture->map->pixels);

		BrPixelmapFree(texture);
	
		BrPSPrimColour0Set(&start[0], 128, 128, 128);
		BrPSPrimColour0Set(&start[1], 128, 128, 128);

		BrPSPrimVertex0Set(&start[0], 76, 36);
		BrPSPrimVertex1Set(&start[0], 244, 36);
		BrPSPrimVertex2Set(&start[0], 76, 204);
		BrPSPrimVertex3Set(&start[0], 244, 204);

		BrPSPrimVertex0Set(&start[1], 76, 36);
		BrPSPrimVertex1Set(&start[1], 244, 36);
		BrPSPrimVertex2Set(&start[1], 76, 204);
		BrPSPrimVertex3Set(&start[1], 244, 204);

		setTPage(&start[0], 1, 0, 512, 256);
		setTPage(&start[1], 1, 0, 512, 256);

		setClut(&start[0], 512, 425);
		setClut(&start[1], 512, 425);
	 
		BrPSPrimMap4Set(&start[0], 0, 0, 168, 0, 0, 168, 168, 168);
		BrPSPrimMap4Set(&start[1], 0, 0, 168, 0, 0, 168, 168, 168);

		BrPSPrimPolyFT4Set(&start[0]);
		BrPSPrimPolyFT4Set(&start[1]);

		PSGfxVSyncCallbackSet(VBL_cbfn);
	}

	BrMatrix3tIdentity(&rotation);

	/*
	 * load pixelmap (64x64 8 bit clut)
	 */
	if((pm = BrPixelmapLoad("test.pix")) == NULL)
		BR_ERROR0("Unable to load test.pix from host");
		
	/*
	 * copy pixelmap to VRAM
	 */
	texture = BrPixelmapAllocateSub(screen, 0, 256, pm->width, pm->height);
	texture->identifier = BrMemStrDup(pm->identifier);
        BrPixelmapRectangleCopy(texture, -texture->origin_x, -texture->origin_y, pm, -pm->origin_x, -pm->origin_y, pm->width, pm->height);

	switch(pm->type) {

		case BR_PMT_INDEX_4:
			tpage_type = 0;
			
			goto copy_clut_0;
			break;

		case BR_PMT_INDEX_8:
			tpage_type = 1;

			/*
			 * perform some suitable validation of the palette
			 */
copy_clut_0:		if(pm->map == NULL)
				BR_ERROR1("Pixlemap '%s' has no palette", pm->identifier);

			if(pm->map->type != BR_PMT_BGR_555)
				BR_ERROR1("Pixelmap '%s' has palette incorrect palette type", pm->identifier);

			if(pm->map->width < pm->map->height)
				BR_ERROR1("Pixelmap '%s' has high rather than wide palette", pm->identifier);

			/*
			 * copy CLUT to VRAM
			 */
                        BrPixelmapRectangleCopy(screen, 256-screen->origin_x, 256-screen->origin_y, pm->map, -pm->map->origin_x, -pm->map->origin_y, pm->map->width, pm->map->height);
			BrPixelmapClutSet(texture, 256, 256);
			
	
			break;

		case BR_PMT_BGR_555:
			tpage_type = 2;
			break;

		default:
			BR_ERROR2("Unable to render with pixelmap '%s' type %d", pm->identifier, pm->type);
	}
	
	BrPixelmapTPageSet(texture, tpage_type);

	BrMapAdd(texture);

	/*
	 * free some memory
	 */
	BrPixelmapFree(pm);
	
	/*
	 * load pixelmap (64x64 8 bit clut) - cursor
	 */
	if((pm = BrPixelmapLoad("cursor.pix")) == NULL)
		BR_ERROR0("Unable to load cursor.pix from host");
		
	/*
	 * copy pixelmap to VRAM
	 */
	texture = BrPixelmapAllocateSub(screen, 512, 256, pm->width, pm->height);
	texture->identifier = BrMemStrDup(pm->identifier);
        BrPixelmapRectangleCopy(texture, -texture->origin_x, -texture->origin_y, pm, -pm->origin_x, -pm->origin_y, pm->width, pm->height);

	switch(pm->type) {

		case BR_PMT_INDEX_4:
			tpage_type = 0;
			
			goto copy_clut_1;
			break;

		case BR_PMT_INDEX_8:
			tpage_type = 1;

			/*
			 * perform some suitable validation of the palette
			 */
copy_clut_1:		if(pm->map == NULL)
				BR_ERROR1("Pixlemap '%s' has no palette", pm->identifier);

			if(pm->map->type != BR_PMT_BGR_555)
				BR_ERROR1("Pixelmap '%s' has palette incorrect palette type", pm->identifier);

			if(pm->map->width < pm->map->height)
				BR_ERROR1("Pixelmap '%s' has high rather than wide palette", pm->identifier);

			/*
			 * copy CLUT to VRAM
			 */
                        BrPixelmapRectangleCopy(screen, 256-screen->origin_x, 257-screen->origin_y, pm->map, -pm->map->origin_x, -pm->map->origin_y, pm->map->width, pm->map->height);
			BrPixelmapClutSet(texture, 256, 257);
			
			break;

		case BR_PMT_BGR_555:
			tpage_type = 2;
			break;

		default:
			BR_ERROR2("Unable to render with pixelmap '%s' type %d", pm->identifier, pm->type);
	}
	
	BrPixelmapTPageSet(texture, tpage_type);

	BrMapAdd(texture);

	/*
	 * free some memory
	 */
	BrPixelmapFree(pm);

	/*
	 * load some materials from script
	 */
	i = BrFmtScriptMaterialLoadMany("material.scr", mats, 32);
	BrMaterialAddMany(mats, i);

	/*
	 * load model
 	 */
        model = BrModelLoad("tcube.dat");

	if(model == NULL)
		BR_ERROR0("Unable to load test.dat from host");

	model->flags &= ~BR_MODF_CUSTOM;
	model->flags |= BR_MODF_KEEP_ORIGINAL;

	material = BrMaterialFind("tex_0");

	for(i=0; i<model->nfaces; i++)
		model->faces[i].material = material;

	BrModelAdd(model);

#if 0
	model->flags |= BR_MODF_QUICK_UPDATE;
#endif
	
	/*
	 * perpare cursor
	 */
	pm = BrMapFind("cursor.pix");
	{
		brps_rectangle rect;

		BrPSRectangleSet(&rect, 0, 0, 255, 255);
		BrPSPrimPolyMFT4Set(&cursor[0], 0, 0, pm->tpage, &rect);
		BrPSPrimPolyMFT4Set(&cursor[1], 0, 0, pm->tpage, &rect);

		cursor[0].poly.tpage = pm->tpage;
		cursor[1].poly.tpage = pm->tpage;

		BrPSPrimMap4Set(&cursor[0].poly, 0, 0, 63, 0, 0, 63, 63, 63);
		BrPSPrimMap4Set(&cursor[1].poly, 0, 0, 63, 0, 0, 63, 63, 63);

		cursor[0].poly.clut = pm->clut;
		cursor[1].poly.clut = pm->clut;


		BrPSPrimVertex0Set(&cursor[0].poly, cursor_x - 32, cursor_y - 32);
		BrPSPrimVertex1Set(&cursor[0].poly, cursor_x + 32, cursor_y - 32);
		BrPSPrimVertex2Set(&cursor[0].poly, cursor_x - 32, cursor_y + 32);
		BrPSPrimVertex3Set(&cursor[0].poly, cursor_x + 32, cursor_y + 32);

		BrPSPrimVertex0Set(&cursor[1].poly, cursor_x - 32, cursor_y - 32);
		BrPSPrimVertex1Set(&cursor[1].poly, cursor_x + 32, cursor_y - 32);
		BrPSPrimVertex2Set(&cursor[1].poly, cursor_x - 32, cursor_y + 32);
		BrPSPrimVertex3Set(&cursor[1].poly, cursor_x + 32, cursor_y + 32);

		BrPSPrimColour0Set(&cursor[0].poly, 128, 128, 128);
		BrPSPrimColour0Set(&cursor[1].poly, 128, 128, 128);

		BrPSPrimTerminate(&cursor[0].mode);
		BrPSPrimTerminate(&cursor[1].mode);
	}

	/*
	 * reset VBL callback
	 */
	PSGfxVSyncCallbackSet(NULL);

	/*
	 * allocate BR_ACTOR_MODEL, uses inherited root order table
	 */
	actor = BrActorAdd(root, BrActorAllocate(BR_ACTOR_MODEL, NULL));
	actor->t.type = BR_TRANSFORM_MATRIX3T;
	BrMatrix3tTranslate(&actor->t.t.mat3t.mat, xpos, ypos, zpos);

	actor->model = model;

	/*
	 * Remember a pointer to a material for testing pick2d
	 */
	PickMaterial[0] = BrMaterialFind("green");
	PickMaterial[1] = model->faces[0].material;
}

/*
 * BRender Rendering function
 */
br_int_8 Sample12_RenderFn(void)
{
	br_uint_8 db_index = PSGfxDoubleBufferIndexRead();
    
        BrZsSceneRender(root, camera, back_screen[db_index]);

	DrawOTag((br_uint_32 *)&cursor[db_index].mode);
}

/*
 * data Update function
 */
br_int_8 Sample12_UpdateFn(void)
{
	counter = (counter >= 360)?counter - 360:counter + 2;
		
	BrMatrix3tTranslate(&actor->t.t.mat3t.mat ,xpos, ypos, zpos);

	BrMatrix3tPre(&actor->t.t.mat3t.mat, &rotation);

	if(animate)
		BrMatrix3tPreRotateY(&actor->t.t.mat3t.mat, BR_ANGLE_DEG(counter));
}

int BR_CALLBACK BrPickNearestModelCallback(
		br_model *model,
		br_material *material,
		br_vector3 *ray_pos, br_vector3 *ray_dir,
		br_scalar t,
		int f,
		int e,
		int v,
		br_vector3 *p,
		br_vector2 *map,
		br_pick_nearest *pn)
{
	if(t < pn->t) {
		pn->t = t;
		pn->actor = pn->temp_actor;
		pn->model = model;
		pn->material = material;
		pn->point = *p;
		pn->face = f;
		pn->edge = e;
		pn->vertex = v;
		pn->map = *map;
	}

	return 0;
}
/*
 * Test callback
 */
int BR_CALLBACK BrPickNearestCallback(
		br_actor *actor,
		br_model *model,
		br_material *material,
		br_vector3 *ray_pos, br_vector3 *ray_dir,
		br_scalar t_near, br_scalar t_far,
		br_pick_nearest *pn)
{
	pn->temp_actor = actor;

	BrModelPick2D(model, material, ray_pos, ray_dir, t_near, t_far,
		(br_modelpick2d_cbfn *)BrPickNearestModelCallback, pn);

	return 0;
}



/*
 * Pad test
 */
br_int_8 Sample12_PadTest(void)
{
	int ret = 0;

	unsigned long  padd = PadRead(1);

	if(PAD_COMPARE(padd, PAD_UP)) ypos = BR_ADD(ypos,BR_SCALAR(0.05));
	if(PAD_COMPARE(padd, PAD_DOWN)) ypos = BR_SUB(ypos,BR_SCALAR(0.05));

	if(PAD_COMPARE(padd, PAD_LEFT)) xpos = BR_SUB(xpos,BR_SCALAR(0.05));
	if(PAD_COMPARE(padd, PAD_RIGHT)) xpos = BR_ADD(xpos,BR_SCALAR(0.05));

	if(PAD_COMPARE(padd, PAD_TRI)) BrMatrix3tPreRotateX(&rotation,BR_ANGLE_DEG(2.0));
	if(PAD_COMPARE(padd, PAD_CROSS)) BrMatrix3tPreRotateX(&rotation,BR_ANGLE_DEG(-2.0));

	if(PAD_COMPARE(padd, PAD_FRONT_LTOP)) {

	    	if(PAD_COMPARE(padd, PAD_SQUARE)) BrMatrix3tPreRotateZ(&rotation, BR_ANGLE_DEG(2.0));
		if(PAD_COMPARE(padd, PAD_CIRCLE)) BrMatrix3tPreRotateZ(&rotation, BR_ANGLE_DEG(-2.0));
	} else {

		if(PAD_COMPARE(padd, PAD_SQUARE)) BrMatrix3tPreRotateY(&rotation,BR_ANGLE_DEG(-2.0));
		if(PAD_COMPARE(padd, PAD_CIRCLE)) BrMatrix3tPreRotateY(&rotation,BR_ANGLE_DEG(2.0));
	}

	if(PAD_COMPARE(padd, PAD_FRONT_RTOP)) {
	    	zpos = BR_ADD(zpos,BR_SCALAR(0.1));
	}

	if(PAD_COMPARE(padd, PAD_FRONT_RBOT)) {
		zpos = BR_SUB(zpos,BR_SCALAR(0.1));
	}

	if(PAD_COMPARE(padd, PAD_FRONT_LBOT)) {
	    	animate ^= 1;

		while(PAD_COMPARE(PadRead(1), PAD_FRONT_LBOT));
	}


	if(PAD_COMPARE(padd, PAD_SELECT)) {

		br_uint_8 db_index = PSGfxDoubleBufferIndexRead();

		pickNearest.model = NULL;
		pickNearest.t = BR_SCALAR_MAX;
	    
		BrScenePick2D(root,camera,
			back_screen[db_index],
			(back_screen[db_index]->width / 2) - cursor_x,
			(back_screen[db_index]->height / 2) - cursor_y,
			(br_pick2d_cbfn *)BrPickNearestCallback,
			&pickNearest);

		if(pickNearest.model) {

			if(pickNearest.model->faces[pickNearest.face].material != PickMaterial[0])
				pickNearest.model->faces[pickNearest.face].material = PickMaterial[0];
			else
				pickNearest.model->faces[pickNearest.face].material = PickMaterial[1];

			BrModelUpdate(pickNearest.model,BR_MODU_ALL);
		}

		while(PAD_COMPARE(PadRead(1), PAD_SELECT));
	}

	return ret;
}
