/* Copyright (c) 1993-1996 Argonaut Technologies Limited. All rights reserved.
 *
 * $Id: SAMPLE13.C 1.1 1996/09/12 14:01:20 NIKH Exp $
 * $Locker: $
 *
 * PS Generic test harness sample
 *
 * Sample13 - textured model
 * model 'test.dat', pixelmap 'test.pix', order table inherited from root, one light
 */

/*
 * Sony library headers
 */
#include <sys/types.h>
#include <r3000.h>
#include <asm.h>
#include <kernel.h>
#include <libgte.h>
#include <libgpu.h>
#include <libetc.h>

/*
 * BRender headers
 */
#include "brender.h"
#include "psio.h"

/*
 * test harness header
 */
#include "harness.h"

/*
 * test harness generic functions for samples
 */
#include "support.h"

/*
 * Sample13 prototypes
 */
br_int_8 Sample13_InitWorld(void);
br_int_8 Sample13_RenderFn(void);
br_int_8 Sample13_UpdateFn(void);
br_int_8 Sample13_PadTest(void);


_sample_fn_list Sample13_fn_table = {

	_GenericInitDevice,			/* init PS hardware, BRender & PSGfx */
	Sample13_InitWorld,			/* init data structures */
	Sample13_RenderFn,			/* rendering function */
	_GenericDoubleBuffer,			/* double buffer function */
	Sample13_UpdateFn,			/* update world */
	NULL,					/* double buffer callback */
	_GenericLoopTest,			/* loop decision */
	Sample13_PadTest,			/* joypad */
	_GenericTerminate,			/* terminate sample (shut down BRender, PSGfx) */
	
};

/*
 * setup test harness control table
 */
_sample_fn_list *_sample_fn_table = &Sample13_fn_table;

/*
 * Sample13 data
 */
br_scalar xpos = 0, ypos = 0, zpos = BR_SCALAR(-4);
br_matrix3t rotation;
br_actor *actor;


int counter = 0, animate = 1;

brps_prim_poly_ft4 start[2];

void VBL_cbfn(br_uint_8 db_index)
{
	br_uint_32 *db_table;
	br_order_table *ot;
	
        BrZsSceneRender(root, camera, back_screen[db_index]);

	ot = BrZsActorOrderTableGet(root);
	    
	db_table = ((br_uint_32 *)ot->table[db_index]) + 1;
	BrPSPrimAdd(db_table, &start[db_index]);

	BrPixelmapDoubleBuffer(screen, back_screen[db_index]);
}
 
/*
 * setup data structures
 */
br_int_8 Sample13_InitWorld(void)
{
	br_pixelmap *pm, *texture;
	br_model *model;
	br_material *mats[32], *material;
	
	int i, tpage_type;
    
	BrMatrix3tIdentity(&rotation);

	/*
	 * load pixelmap (64x64 8 bit clut)
	 */
	if((pm = BrPixelmapLoad("test.pix")) == NULL)
		BR_ERROR0("Unable to load test.pix from host");
		
	if(pm->map == NULL)
		BR_ERROR1("Pixelmap %s has no palette", pm->identifier);
		
	/*
	 * copy pixelmap to VRAM
	 */
	texture = BrPixelmapAllocateSub(screen, 0, 256, pm->width, pm->height);
	texture->identifier = BrMemStrDup(pm->identifier);
        BrPixelmapRectangleCopy(texture, -texture->origin_x, -texture->origin_y, pm, -pm->origin_x, -pm->origin_y, pm->width, pm->height);

	switch(pm->type) {

		case BR_PMT_INDEX_4:
			tpage_type = 0;
			
			goto copy_clut;
			break;

		case BR_PMT_INDEX_8:
			tpage_type = 1;

			/*
			 * perform some suitable validation of the palette
			 */
copy_clut:		if(pm->map == NULL)
				BR_ERROR1("Pixlemap '%s' has no palette", pm->identifier);

			if(pm->map->type != BR_PMT_BGR_555)
				BR_ERROR1("Pixelmap '%s' has palette incorrect palette type", pm->identifier);

			if(pm->map->width < pm->map->height)
				BR_ERROR1("Pixelmap '%s' has high rather than wide palette", pm->identifier);

			/*
			 * copy CLUT to VRAM
			 */
                        BrPixelmapRectangleCopy(screen, 256-screen->origin_x, 256-screen->origin_y, pm->map, -pm->map->origin_x, -pm->map->origin_y, pm->map->width, pm->map->height);
			BrPixelmapClutSet(texture, 256, 256);
			
	
			break;

		case BR_PMT_BGR_555:
			tpage_type = 2;
			break;

		default:
			BR_ERROR2("Unable to render with pixelmap '%s' type %d", pm->identifier, pm->type);
	}
	
	BrPixelmapTPageSet(texture, tpage_type);

	BrMapAdd(texture);

	/*
	 * free some memory
	 */
	BrPixelmapFree(pm);

	/*
	 * load some materials from script
	 */
	i = BrFmtScriptMaterialLoadMany("material.scr", mats, 32);
	BrMaterialAddMany(mats, i);

	/*
	 * load model
	 */
	model = BrModelLoad("tcube.dat");
	 
	if(model == NULL)
		BR_ERROR0("Unable to load tcube.dat from host");

	material = BrMaterialFind("tex_0");

	for(i=0; i<model->nfaces; i++)
		model->faces[i].material = material;

	model->flags &= ~BR_MODF_CUSTOM;
	model->flags |= BR_MODF_KEEP_ORIGINAL;

	BrModelAdd(model);

	/*
	 * sneak some VBL interrupt code in here while saving data...
	 */
	{
		brps_rectangle rect;

		texture = BrPixelmapLoad("pssml.pix");
			    
		BrPSRectangleSet(&rect, 512, 256, texture->width/2, texture->height);
		LoadImage((RECT *)&rect, (unsigned long *)texture->pixels);


		BrPSRectangleSet(&rect, 512, 425, 256, 1);
		LoadImage((RECT *)&rect, (unsigned long *)texture->map->pixels);

		BrPixelmapFree(texture);
	
		BrPSPrimColour0Set(&start[0], 128, 128, 128);
		BrPSPrimColour0Set(&start[1], 128, 128, 128);

		BrPSPrimVertex0Set(&start[0], 76, 36);
		BrPSPrimVertex1Set(&start[0], 244, 36);
		BrPSPrimVertex2Set(&start[0], 76, 204);
		BrPSPrimVertex3Set(&start[0], 244, 204);

		BrPSPrimVertex0Set(&start[1], 76, 36);
		BrPSPrimVertex1Set(&start[1], 244, 36);
		BrPSPrimVertex2Set(&start[1], 76, 204);
		BrPSPrimVertex3Set(&start[1], 244, 204);

		setTPage(&start[0], 1, 0, 512, 256);
		setTPage(&start[1], 1, 0, 512, 256);

		setClut(&start[0], 512, 425);
		setClut(&start[1], 512, 425);
	 
		BrPSPrimMap4Set(&start[0], 0, 0, 168, 0, 0, 168, 168, 168);
		BrPSPrimMap4Set(&start[1], 0, 0, 168, 0, 0, 168, 168, 168);

		BrPSPrimPolyFT4Set(&start[0]);
		BrPSPrimPolyFT4Set(&start[1]);

		PSGfxVSyncCallbackSet(VBL_cbfn);
	}
	/*
	 * save with prepared data
	 */
	model->flags &= BR_MODF_PREPREPARED;
	 
	BrModelSave("save.dat", model);

	BrModelRemove(model);
	BrModelFree(model);

	/*
	 * reload prepared model
	 */
	model = BrModelLoad("save.dat");

	BrModelAdd(model);	/* this should not take long as model is already prepared */

	/*
	 * reset VBL callback
	 */
	PSGfxVSyncCallbackSet(NULL);

	/*
	 * allocate BR_ACTOR_MODEL, uses inherited root order table
	 */
	actor = BrActorAdd(root, BrActorAllocate(BR_ACTOR_MODEL, NULL));
	actor->t.type = BR_TRANSFORM_MATRIX3T;
	BrMatrix3tTranslate(&actor->t.t.mat3t.mat, xpos, ypos, zpos);

	actor->model = model;
}

/*
 * BRender Rendering function
 */
br_int_8 Sample13_RenderFn(void)
{
	br_uint_8 db_index = PSGfxDoubleBufferIndexRead();
    
        BrZsSceneRender(root, camera, back_screen[db_index]);
}

/*
 * data Update function
 */
br_int_8 Sample13_UpdateFn(void)
{
	counter = (counter >= 360)?counter - 360:counter + 2;
		
	BrMatrix3tTranslate(&actor->t.t.mat3t.mat ,xpos, ypos, zpos);

	BrMatrix3tPre(&actor->t.t.mat3t.mat, &rotation);

	if(animate)
		BrMatrix3tPreRotateY(&actor->t.t.mat3t.mat, BR_ANGLE_DEG(counter));
}

/*
 * Pad test
 */
br_int_8 Sample13_PadTest(void)
{
	int ret = 0;

	unsigned long  padd = PadRead(1);

	if(PAD_COMPARE(padd, PAD_UP)) ypos = BR_ADD(ypos,BR_SCALAR(0.05));
	if(PAD_COMPARE(padd, PAD_DOWN)) ypos = BR_SUB(ypos,BR_SCALAR(0.05));

	if(PAD_COMPARE(padd, PAD_LEFT)) xpos = BR_SUB(xpos,BR_SCALAR(0.05));
	if(PAD_COMPARE(padd, PAD_RIGHT)) xpos = BR_ADD(xpos,BR_SCALAR(0.05));

	if(PAD_COMPARE(padd, PAD_TRI)) BrMatrix3tPreRotateX(&rotation,BR_ANGLE_DEG(2.0));
	if(PAD_COMPARE(padd, PAD_CROSS)) BrMatrix3tPreRotateX(&rotation,BR_ANGLE_DEG(-2.0));

	if(PAD_COMPARE(padd, PAD_FRONT_LTOP)) {

	    	if(PAD_COMPARE(padd, PAD_SQUARE)) BrMatrix3tPreRotateZ(&rotation, BR_ANGLE_DEG(2.0));
		if(PAD_COMPARE(padd, PAD_CIRCLE)) BrMatrix3tPreRotateZ(&rotation, BR_ANGLE_DEG(-2.0));
	} else {

		if(PAD_COMPARE(padd, PAD_SQUARE)) BrMatrix3tPreRotateY(&rotation,BR_ANGLE_DEG(-2.0));
		if(PAD_COMPARE(padd, PAD_CIRCLE)) BrMatrix3tPreRotateY(&rotation,BR_ANGLE_DEG(2.0));
	}

	if(PAD_COMPARE(padd, PAD_FRONT_RTOP)) {
	    	zpos = BR_ADD(zpos,BR_SCALAR(0.5));
	}

	if(PAD_COMPARE(padd, PAD_FRONT_RBOT)) {
		zpos = BR_SUB(zpos,BR_SCALAR(0.5));
	}

	if(PAD_COMPARE(padd, PAD_FRONT_LBOT)) {
	    	animate ^= 1;

		while(PAD_COMPARE(PadRead(1), PAD_FRONT_LBOT));
	}

	return ret;
}
