/* Copyright (c) 1993-1996 Argonaut Technologies Limited. All rights reserved.
 *
 * $Id: SAMPLE14.C 1.1 1996/09/12 14:01:21 NIKH Exp $
 * $Locker: $
 *
 * PS Generic test harness sample
 *
 * Sample14 - textured model
 * model 'test.dat', pixelmap 'test.pix', order table inherited from root, one light
 */

/*
 * Sony library headers
 */
#include <sys/types.h>
#include <r3000.h>
#include <asm.h>
#include <kernel.h>
#include <libgte.h>
#include <libgpu.h>
#include <libetc.h>

/*
 * BRender headers
 */
#include "brender.h"
#include "psio.h"

/*
 * test harness header
 */
#include "harness.h"

/*
 * test harness generic functions for samples
 */
#include "support.h"

/*
 * Sample14 prototypes
 */
br_int_8 Sample14_InitWorld(void);
br_int_8 Sample14_RenderFn(void);
br_int_8 Sample14_UpdateFn(void);
br_int_8 Sample14_PadTest(void);


_sample_fn_list Sample14_fn_table = {

	_GenericInitDevice,			/* init PS hardware, BRender & PSGfx */
	Sample14_InitWorld,			/* init data structures */
	Sample14_RenderFn,			/* rendering function */
	_GenericDoubleBuffer,			/* double buffer function */
	Sample14_UpdateFn,			/* update world */
	NULL,					/* double buffer callback */
	_GenericLoopTest,			/* loop decision */
	Sample14_PadTest,			/* joypad */
	_GenericTerminate,			/* terminate sample (shut down BRender, PSGfx) */
	
};

/*
 * setup test harness control table
 */
_sample_fn_list *_sample_fn_table = &Sample14_fn_table;

/*
 * Sample14 data
 */
br_scalar xpos = 0, ypos = 0, zpos = BR_SCALAR(-4);
br_matrix3t rotation;
br_actor *actor;


int counter = 0, animate = 1;

brps_prim_poly_ft4 start[2];

void VBL_cbfn(br_uint_8 db_index)
{
	br_uint_32 *db_table;
	br_order_table *ot;
	
        BrZsSceneRender(root, camera, back_screen[db_index]);

	ot = BrZsActorOrderTableGet(root);
	    
	db_table = ((br_uint_32 *)ot->table[db_index]) + 1;
	BrPSPrimAdd(db_table, &start[db_index]);

	BrPixelmapDoubleBuffer(screen, back_screen[db_index]);
}
 
/*
 * setup data structures
 */
br_int_8 Sample14_InitWorld(void)
{
        br_order_table *order_table;
    	br_actor *actor_model;
	br_pixelmap *pm, *texture;
	br_model *model;
	br_material *mats[32], *material;
	
	int i, tpage_type;
    
	BrMatrix3tIdentity(&rotation);

	/*
	 * load pixelmap (64x64 8 bit clut)
	 */
	if((pm = BrPixelmapLoad("test.pix")) == NULL)
		BR_ERROR0("Unable to load test.pix from host");
		
	/*
	 * copy pixelmap to VRAM
	 */
	texture = BrPixelmapAllocateSub(screen, 0, 256, pm->width, pm->height);
	texture->identifier = BrMemStrDup(pm->identifier);
        BrPixelmapRectangleCopy(texture, -texture->origin_x, -texture->origin_y, pm, -pm->origin_x, -pm->origin_y, pm->width, pm->height);

	switch(pm->type) {

		case BR_PMT_INDEX_4:
			tpage_type = 0;
			
			goto copy_clut;
			break;

		case BR_PMT_INDEX_8:
			tpage_type = 1;

			/*
			 * perform some suitable validation of the palette
			 */
copy_clut:		if(pm->map == NULL)
				BR_ERROR1("Pixlemap '%s' has no palette", pm->identifier);

			if(pm->map->type != BR_PMT_BGR_555)
				BR_ERROR1("Pixelmap '%s' has palette incorrect palette type", pm->identifier);

			if(pm->map->width < pm->map->height)
				BR_ERROR1("Pixelmap '%s' has high rather than wide palette", pm->identifier);

			/*
			 * copy CLUT to VRAM
			 */
                        BrPixelmapRectangleCopy(screen, 512-screen->origin_x, 256-screen->origin_y, pm->map, -pm->map->origin_x, -pm->map->origin_y, pm->map->width, pm->map->height);
                        BrPixelmapClutSet(texture, 512, 256);
			
			break;

		case BR_PMT_BGR_555:
			tpage_type = 2;
			break;

		default:
			BR_ERROR2("Unable to render with pixelmap '%s' type %d", pm->identifier, pm->type);
	}
	
	BrPixelmapTPageSet(texture, tpage_type);

	BrMapAdd(texture);

	/*
	 * free some memory
	 */
	BrPixelmapFree(pm);
	
	if((pm = BrPixelmapLoad("test2.pix")) == NULL)
		BR_ERROR0("Unable to load test2.pix from host");
		
	/*
	 * copy pixelmap to VRAM
	 */
	texture = BrPixelmapAllocateSub(screen, 256, 256, pm->width, pm->height);
	texture->identifier = BrMemStrDup(pm->identifier);
        BrPixelmapRectangleCopy(texture, -texture->origin_x, -texture->origin_y, pm, -pm->origin_x, -pm->origin_y, pm->width, pm->height);

	switch(pm->type) {

		case BR_PMT_INDEX_4:
			tpage_type = 0;
			
			goto copy_clut2;
			break;

		case BR_PMT_INDEX_8:
			tpage_type = 1;

			/*
			 * perform some suitable validation of the palette
			 */
copy_clut2:		if(pm->map == NULL)
				BR_ERROR1("Pixlemap '%s' has no palette", pm->identifier);

			if(pm->map->type != BR_PMT_BGR_555)
				BR_ERROR1("Pixelmap '%s' has palette incorrect palette type", pm->identifier);

			if(pm->map->width < pm->map->height)
				BR_ERROR1("Pixelmap '%s' has high rather than wide palette", pm->identifier);

			/*
			 * copy CLUT to VRAM
			 */
                        BrPixelmapRectangleCopy(screen, 512-screen->origin_x, 257-screen->origin_y, pm->map, -pm->map->origin_x, -pm->map->origin_y, pm->map->width, pm->map->height);
			BrPixelmapClutSet(texture, 512, 257);
			
			break;

		case BR_PMT_BGR_555:
			tpage_type = 2;
			break;

		default:
			BR_ERROR2("Unable to render with pixelmap '%s' type %d", pm->identifier, pm->type);
	}
	
	BrPixelmapTPageSet(texture, tpage_type);

	BrMapAdd(texture);

	/*
	 * free some memory
	 */
	BrPixelmapFree(pm);

	/*
	 * load some materials from script
	 */
	i = BrFmtScriptMaterialLoadMany("material.scr", mats, 32);
	BrMaterialAddMany(mats, i);

	/*
	 * load model
	 */

	if((model = BrModelLoad("test.dat")) == NULL)
		BR_ERROR0("Unable to load 'test.dat' from host");

	material = BrMaterialFind("tex_1");

	for(i=0; i<model->nfaces; i++)
		model->faces[i].material = material;

	BrModelAdd(model);

	if((model = BrModelLoad("tcube.dat")) == NULL)
		BR_ERROR0("Unable to load tcube.dat from host");

	material = BrMaterialFind("tex_0");
	for(i=0; i<model->nfaces; i++)
		model->faces[i].material = material;

	model->flags &= ~BR_MODF_CUSTOM;
	model->flags |= BR_MODF_KEEP_ORIGINAL;

	BrModelAdd(model);

	/*
	 * allocate BR_ACTOR_BOUNDS, set order table
	 */
	actor = BrActorAdd(root, BrActorAllocate(BR_ACTOR_BOUNDS, NULL));
	BrMatrix3tTranslate(&actor->t.t.mat3t.mat, xpos, ypos, zpos);

        order_table = BrZsOrderTableAllocate(8192, BR_ORDER_TABLE_LEAVE_BOUNDS | BR_ORDER_TABLE_SORT_CENTRE, BR_SORT_AVERAGE);
        order_table->min_z = 0;
        order_table->max_z = 32000;
        BrZsActorOrderTableSet(actor, order_table);

	/*
	 * setup some models - all will be rendered into the above order table
	 */
	model = BrModelFind("tcube");
	 
	actor_model = BrActorAdd(actor, BrActorAllocate(BR_ACTOR_MODEL, NULL));
	BrMatrix3tTranslate(&actor_model->t.t.mat3t.mat, BR_SCALAR(2.0), 0, 0);
	actor_model->model = model;

	actor_model = BrActorAdd(actor, BrActorAllocate(BR_ACTOR_MODEL, NULL));
	BrMatrix3tTranslate(&actor_model->t.t.mat3t.mat, BR_SCALAR(-2.0), 0, 0);
	actor_model->model = BrModelAdd(BrModelClone(model));

	actor_model = BrActorAdd(actor, BrActorAllocate(BR_ACTOR_MODEL, NULL));
	BrMatrix3tTranslate(&actor_model->t.t.mat3t.mat, 0, BR_SCALAR(2.0), 0);
	actor_model->model = BrModelAdd(BrModelClone(model));
	
	actor_model = BrActorAdd(actor, BrActorAllocate(BR_ACTOR_MODEL, NULL));
	BrMatrix3tTranslate(&actor_model->t.t.mat3t.mat, 0, BR_SCALAR(-2.0), 0);
	actor_model->model = BrModelAdd(BrModelClone(model));
	
	actor_model = BrActorAdd(actor, BrActorAllocate(BR_ACTOR_MODEL, NULL));
	BrMatrix3tTranslate(&actor_model->t.t.mat3t.mat, 0, 0, BR_SCALAR(2.0));
	actor_model->model = BrModelAdd(BrModelClone(model));
	
	actor_model = BrActorAdd(actor, BrActorAllocate(BR_ACTOR_MODEL, NULL));
	BrMatrix3tTranslate(&actor_model->t.t.mat3t.mat, 0, 0, BR_SCALAR(-2.0));
	actor_model->model = BrModelAdd(BrModelClone(model));

	/*
	 * Load a second model
	 */
	model = BrModelFind("test");

	actor_model = BrActorAdd(actor, BrActorAllocate(BR_ACTOR_MODEL, NULL));
	actor_model->model = model;

	BrActorToBounds(((br_bounds *)actor->type_data), actor);
}

/*
 * BRender Rendering function
 */
br_int_8 Sample14_RenderFn(void)
{
	br_uint_8 db_index = PSGfxDoubleBufferIndexRead();
    
        BrZsSceneRender(root, camera, back_screen[db_index]);
}

/*
 * data Update function
 */
br_int_8 Sample14_UpdateFn(void)
{
	counter = (counter >= 360)?counter - 360:counter + 2;
		
	BrMatrix3tTranslate(&actor->t.t.mat3t.mat ,xpos, ypos, zpos);

	BrMatrix3tPre(&actor->t.t.mat3t.mat, &rotation);

	if(animate)
		BrMatrix3tPreRotateY(&actor->t.t.mat3t.mat, BR_ANGLE_DEG(counter));
}

/*
 * Pad test
 */
br_int_8 Sample14_PadTest(void)
{
	int ret = 0;

	unsigned long  padd = PadRead(1);

	if(PAD_COMPARE(padd, PAD_UP)) ypos = BR_ADD(ypos,BR_SCALAR(0.05));
	if(PAD_COMPARE(padd, PAD_DOWN)) ypos = BR_SUB(ypos,BR_SCALAR(0.05));

	if(PAD_COMPARE(padd, PAD_LEFT)) xpos = BR_SUB(xpos,BR_SCALAR(0.05));
	if(PAD_COMPARE(padd, PAD_RIGHT)) xpos = BR_ADD(xpos,BR_SCALAR(0.05));

	if(PAD_COMPARE(padd, PAD_TRI)) BrMatrix3tPreRotateX(&rotation,BR_ANGLE_DEG(2.0));
	if(PAD_COMPARE(padd, PAD_CROSS)) BrMatrix3tPreRotateX(&rotation,BR_ANGLE_DEG(-2.0));

	if(PAD_COMPARE(padd, PAD_FRONT_LTOP)) {

	    	if(PAD_COMPARE(padd, PAD_SQUARE)) BrMatrix3tPreRotateZ(&rotation, BR_ANGLE_DEG(2.0));
		if(PAD_COMPARE(padd, PAD_CIRCLE)) BrMatrix3tPreRotateZ(&rotation, BR_ANGLE_DEG(-2.0));
	} else {

		if(PAD_COMPARE(padd, PAD_SQUARE)) BrMatrix3tPreRotateY(&rotation,BR_ANGLE_DEG(-2.0));
		if(PAD_COMPARE(padd, PAD_CIRCLE)) BrMatrix3tPreRotateY(&rotation,BR_ANGLE_DEG(2.0));
	}

	if(PAD_COMPARE(padd, PAD_FRONT_RTOP)) {
	    	zpos = BR_ADD(zpos,BR_SCALAR(0.5));
	}

	if(PAD_COMPARE(padd, PAD_FRONT_RBOT)) {
		zpos = BR_SUB(zpos,BR_SCALAR(0.5));
	}

	if(PAD_COMPARE(padd, PAD_FRONT_LBOT)) {
	    	animate ^= 1;

		while(PAD_COMPARE(PadRead(1), PAD_FRONT_LBOT));
	}

	if(PAD_COMPARE(padd, PAD_SELECT)) {

		br_pixelmap *pm;
		br_uint_8 db_index = PSGfxDoubleBufferIndexRead();

		/*
		 * screen capture - useful for checking VRAM
		 */
		pm = BrPixelmapAllocate(back_screen[db_index]->type,
					back_screen[db_index]->width,
					back_screen[db_index]->height,
					NULL,
					0);

		BrPixelmapCopy(pm, back_screen[db_index]);

		DrawSync(0);

		BrPixelmapSave("out.pix", pm);

		BrPixelmapFree(pm);

		while(PAD_COMPARE(PadRead(1), PAD_SELECT));
	}

	return ret;
}
