/* Copyright (c) 1993-1996 Argonaut Technologies Limited. All rights reserved.
 *
 * $Id: SAMPLE5.C 1.1 1996/09/12 14:01:17 NIKH Exp $
 * $Locker: $
 *
 * PS Generic test harness sample
 *
 * Sample5 - prelit textured model with tiled texture
 * model 'tile.dat', pixelmap 'test.pix', order table inherited from root, one light
 */

/*
 * Sony library headers
 */
#include <sys/types.h>
#include <r3000.h>
#include <asm.h>
#include <kernel.h>
#include <libgte.h>
#include <libgpu.h>
#include <libetc.h>

/*
 * BRender headers
 */
#include "brender.h"
#include "psio.h"

/*
 * test harness header
 */
#include "harness.h"

/*
 * test harness generic functions for samples
 */
#include "support.h"

/*
 * Sample5 prototypes
 */
br_int_8 Sample5_InitWorld(void);
br_int_8 Sample5_RenderFn(void);
br_int_8 Sample5_UpdateFn(void);
br_int_8 Sample5_PadTest(void);


_sample_fn_list Sample5_fn_table = {

	_GenericInitDevice,			/* init PS hardware, BRender & PSGfx */
	Sample5_InitWorld,			/* init data structures */
	Sample5_RenderFn,			/* rendering function */
	_GenericDoubleBuffer,			/* double buffer function */
	Sample5_UpdateFn,			/* update world */
	NULL,					/* double buffer callback */
	_GenericLoopTest,			/* loop decision */
	Sample5_PadTest,			/* joypad */
	_GenericTerminate,			/* terminate sample (shut down BRender, PSGfx) */
	
};

/*
 * setup test harness control table
 */
_sample_fn_list *_sample_fn_table = &Sample5_fn_table;

/*
 * Sample5 data
 */
br_scalar xpos = 0, ypos = 0, zpos = 0;
br_matrix3t rotation;
br_actor *actor;


int counter = 0, animate = 1;


/*
 * setup data structures
 */
br_int_8 Sample5_InitWorld(void)
{
	br_pixelmap *pm, *texture;
	br_model *model;
	br_material *mats[32];
	
	int i, tpage_type;
    
	BrMatrix3tIdentity(&rotation);

	/*
	 * load pixelmap (64x64 8 bit clut)
	 */
	if((pm = BrPixelmapLoad("test.pix")) == NULL)
		BR_ERROR0("Unable to load test.pix from host");
		
	/*
	 * copy pixelmap to VRAM
	 */
	texture = BrPixelmapAllocateSub(screen, 80, 320, pm->width, pm->height);
	texture->identifier = BrMemStrDup(pm->identifier);
	BrPixelmapRectangleCopy(texture, 0, 0, pm, 0, 0, pm->width, pm->height);

	switch(pm->type) {

		case BR_PMT_INDEX_4:
			tpage_type = 0;
			
			goto copy_clut;
			break;

		case BR_PMT_INDEX_8:
			tpage_type = 1;

			/*
			 * perform some suitable validation of the palette
			 */
copy_clut:		if(pm->map == NULL)
				BR_ERROR1("Pixlemap '%s' has no palette", pm->identifier);

			if(pm->map->type != BR_PMT_BGR_555)
				BR_ERROR1("Pixelmap '%s' has palette incorrect palette type", pm->identifier);

			if(pm->map->width < pm->map->height)
				BR_ERROR1("Pixelmap '%s' has high rather than wide palette", pm->identifier);

			/*
			 * copy CLUT to VRAM
			 */
			BrPixelmapRectangleCopy(screen, 256, 256, pm->map, 0, 0, pm->map->width, pm->map->height);
			BrPixelmapClutSet(texture, 256, 256);
			
	
			break;

		case BR_PMT_BGR_555:
			tpage_type = 2;
			break;

		default:
			BR_ERROR2("Unable to render with pixelmap '%s' type %d", pm->identifier, pm->type);
	}

	BrPixelmapTPageSet(texture, tpage_type);

	BrMapAdd(texture);

	/*
	 * free some memory
	 */
	BrPixelmapFree(pm);

	/*
	 * load some materials from script
	 */
	i = BrFmtScriptMaterialLoadMany("material.scr", mats, 32);
	BrMaterialAddMany(mats, i);

	/*
	 * load model
	 */
	model = BrModelLoad("tile.dat");

	if(model == NULL)
		BR_ERROR0("Unable to load tile.dat from host");

	/*
	 * clear model custom callback flag
	 */
	if(model->flags & BR_MODF_CUSTOM)
		model->flags ^= BR_MODF_CUSTOM;
	model->flags |= BR_MODF_KEEP_ORIGINAL;

	/*
	 * 'prelight' vertices
	 */
	for(i=0; i<model->nvertices; i++) {
	    
	    	model->vertices[i].red = 128;
	    	model->vertices[i].grn = 128;
	    	model->vertices[i].blu = 128;
	}

	BrModelAdd(model);

	BrLightDisable(light);

	/*
	 * allocate BR_ACTOR_MODEL, uses inherited root order table
	 */
	actor = BrActorAdd(root, BrActorAllocate(BR_ACTOR_MODEL, NULL));
	actor->t.type = BR_TRANSFORM_MATRIX3T;
	BrMatrix3tIdentity(&actor->t.t.mat3t.mat);

	actor->model = model;
}

/*
 * BRender Rendering function
 */
br_int_8 Sample5_RenderFn(void)
{
	br_uint_8 db_index = PSGfxDoubleBufferIndexRead();
    
        BrZsSceneRender(root, camera, back_screen[db_index]);
}

/*
 * data Update function
 */
br_int_8 Sample5_UpdateFn(void)
{
	counter = (counter >= 360)?counter - 360:counter + 2;
		
	BrMatrix3tTranslate(&actor->t.t.mat3t.mat ,xpos, ypos, zpos);

	BrMatrix3tPre(&actor->t.t.mat3t.mat, &rotation);

	if(animate)
		BrMatrix3tPreRotateZ(&actor->t.t.mat3t.mat, BR_ANGLE_DEG(counter));
}

/*
 * Pad test
 */
br_int_8 Sample5_PadTest(void)
{
	int ret = 0;

	unsigned long  padd = PadRead(1);

	if(PAD_COMPARE(padd, PAD_UP)) ypos = BR_ADD(ypos,BR_SCALAR(0.05));
	if(PAD_COMPARE(padd, PAD_DOWN)) ypos = BR_SUB(ypos,BR_SCALAR(0.05));

	if(PAD_COMPARE(padd, PAD_LEFT)) xpos = BR_SUB(xpos,BR_SCALAR(0.05));
	if(PAD_COMPARE(padd, PAD_RIGHT)) xpos = BR_ADD(xpos,BR_SCALAR(0.05));

	if(PAD_COMPARE(padd, PAD_TRI)) BrMatrix3tPreRotateX(&rotation,BR_ANGLE_DEG(2.0));
	if(PAD_COMPARE(padd, PAD_CROSS)) BrMatrix3tPreRotateX(&rotation,BR_ANGLE_DEG(-2.0));

	if(PAD_COMPARE(padd, PAD_FRONT_LTOP)) {
	    
	    	if(PAD_COMPARE(padd, PAD_SQUARE)) BrMatrix3tPreRotateZ(&rotation, BR_ANGLE_DEG(2.0));
		if(PAD_COMPARE(padd, PAD_CIRCLE)) BrMatrix3tPreRotateZ(&rotation, BR_ANGLE_DEG(-2.0));
	} else {

		if(PAD_COMPARE(padd, PAD_SQUARE)) BrMatrix3tPreRotateY(&rotation,BR_ANGLE_DEG(-2.0));
		if(PAD_COMPARE(padd, PAD_CIRCLE)) BrMatrix3tPreRotateY(&rotation,BR_ANGLE_DEG(2.0));
	}

	if(PAD_COMPARE(padd, PAD_FRONT_RTOP)) {
	    	zpos = BR_ADD(zpos,BR_SCALAR(0.5));
	}

	if(PAD_COMPARE(padd, PAD_FRONT_RBOT)) {
		zpos = BR_SUB(zpos,BR_SCALAR(0.5));
	}

	if(PAD_COMPARE(padd, PAD_FRONT_LBOT)) {
	    	animate ^= 1;

		while(PAD_COMPARE(PadRead(1), PAD_FRONT_LBOT));
	}

	return ret;
}
