/* Copyright (c) 1993-1996 Argonaut Technologies Limited. All rights reserved.
 *
 * $Id: SAMPLE6.C 1.2 1996/09/12 15:22:29 NIKH Exp $
 * $Locker: $
 *
 * PS Generic test harness sample
 *
 * Sample6 - textured model, double buffer callback, draw sync callback
 * model 'test.dat', pixelmaps 'test.pix' 'back.pix'
 * order table inherited from root, one light
 */

/*
 * Sony library headers
 */
#include <sys/types.h>
#include <r3000.h>
#include <asm.h>
#include <kernel.h>
#include <libgte.h>
#include <libgpu.h>
#include <libetc.h>

/*
 * BRender headers
 */
#include "brender.h"
#include "psio.h"

/*
 * test harness header
 */
#include "harness.h"

/*
 * test harness generic functions for samples
 */
#include "support.h"

/*
 * Sample6 prototypes
 */
br_int_8 Sample6_InitWorld(void);
br_int_8 Sample6_RenderFn(void);
br_int_8 Sample6_UpdateFn(void);
br_int_8 Sample6_PadTest(void);
void BR_CALLBACK Sample6_DoubleBufferCbfn(br_uint_8 db_index);
void BR_CALLBACK Sample6_RenderBoundsCbfn(
	br_actor *actor,
        const br_model *model,
        const br_material *material,
	br_order_table *order_table,
	br_uint_8	style,
        const br_matrix3t *model_to_screen,
        const br_int_32 bounds[4]);


_sample_fn_list Sample6_fn_table = {

	_GenericInitDevice,			/* init PS hardware, BRender & PSGfx */
	Sample6_InitWorld,			/* init data structures */
	Sample6_RenderFn,			/* rendering function */
	_GenericDoubleBuffer,			/* double buffer function */
	Sample6_UpdateFn,			/* update world */
	Sample6_DoubleBufferCbfn,		/* double buffer callback */
	_GenericLoopTest,			/* loop decision */
	Sample6_PadTest,			/* joypad */
	_GenericTerminate,			/* terminate sample (shut down BRender, PSGfx) */
	
};

/*
 * setup test harness control table
 */
_sample_fn_list *_sample_fn_table = &Sample6_fn_table;

/*
 * Sample6 data
 */
br_scalar xpos = 0, ypos = 0, zpos = BR_SCALAR(-3);
br_matrix3t rotation;
br_actor *actor, *actor_model;
br_pixelmap *background;
br_colour outline, fill[2];


int counter = 0, animate = 1;
brps_prim_poly_f4 bounding_box_poly[2][2];
br_int_32 bounding_box[2][4];
int bounding_box_count = 0;
 
/*
 * setup data structures
 */
br_int_8 Sample6_InitWorld(void)
{
	br_pixelmap *pm, *texture;
	br_model *model;
	br_material *mats[32];
	
	int i, x, y, tpage_type;
	br_uint_16 *pixels, pixel;

	outline = BR_COLOUR_RGB(255, 255, 255);
	fill[0] = BR_COLOUR_RGB(80, 255, 80);
	fill[1] = BR_COLOUR_RGB(255, 80, 80);
    
    
	BrMatrix3tIdentity(&rotation);

	/*
	 * load pixelmap (64x64 8 bit clut)
	 */
	if((pm = BrPixelmapLoad("test.pix")) == NULL)
		BR_ERROR0("Unable to load test.pix from host");
		
	/*
	 * copy pixelmap to VRAM
	 */
	texture = BrPixelmapAllocateSub(screen, 0, 256, pm->width, pm->height);
	texture->identifier = BrMemStrDup(pm->identifier);
        BrPixelmapRectangleCopy(texture, -texture->origin_x, -texture->origin_y, pm, -pm->origin_x, -pm->origin_y, pm->width, pm->height);

	switch(pm->type) {

		case BR_PMT_INDEX_4:
			tpage_type = 0;
			
			goto copy_clut;
			break;

		case BR_PMT_INDEX_8:
			tpage_type = 1;

			/*
			 * perform some suitable validation of the palette
			 */
copy_clut:		if(pm->map == NULL)
				BR_ERROR1("Pixlemap '%s' has no palette", pm->identifier);

			if(pm->map->type != BR_PMT_BGR_555)
				BR_ERROR1("Pixelmap '%s' has palette incorrect palette type", pm->identifier);

			if(pm->map->width < pm->map->height)
				BR_ERROR1("Pixelmap '%s' has high rather than wide palette", pm->identifier);

			/*
			 * copy CLUT to VRAM
			 */
                        BrPixelmapRectangleCopy(screen, 256-screen->origin_x, 256-screen->origin_y, pm->map, -pm->map->origin_x, -pm->map->origin_y, pm->map->width, pm->map->height);
			BrPixelmapClutSet(texture, 256, 256);
			
	
			break;

		case BR_PMT_BGR_555:
			tpage_type = 2;
			break;

		default:
			BR_ERROR2("Unable to render with pixelmap '%s' type %d", pm->identifier, pm->type);
	}
	
	BrPixelmapTPageSet(texture, tpage_type);

	BrMapAdd(texture);

	/*
	 * free some memory
	 */
	BrPixelmapFree(pm);

	/*
	 * load some materials from script
	 */
	i = BrFmtScriptMaterialLoadMany("material.scr", mats, 32);
	BrMaterialAddMany(mats, i);

	/*
	 * load background pixelmap (320x256 RGB 555)
	 */
	if((pm = BrPixelmapLoad("back.pix")) == NULL)
		BR_ERROR0("Unable to load back.pix from host");

	background = BrPixelmapAllocateSub(screen, 512, 256, pm->width, pm->height);

	/*
	 * copy to VRAM
	 */
	BrPixelmapCopy(background, pm);

	/*
	 * Free some memory
	 */
	BrMemFree(pm);

	/*
	 * load model
	 */
	model = BrModelLoad("test.dat");

	if(model == NULL)
		BR_ERROR0("Unable to load test.dat from host");

	model->flags &= ~BR_MODF_CUSTOM;
	model->flags |= BR_MODF_KEEP_ORIGINAL;

	BrModelAdd(model);

	/*
	 * dummy actor head
	 */
	actor = BrActorAdd(root, BrActorAllocate(BR_ACTOR_NONE, NULL));
	actor->t.type = BR_TRANSFORM_MATRIX3T;
	BrMatrix3tIdentity(&actor->t.t.mat3t.mat);


	/*
	 * allocate BR_ACTOR_MODEL, uses inherited root order table
	 */
	actor_model = BrActorAdd(actor, BrActorAllocate(BR_ACTOR_MODEL, NULL));
	actor_model->t.type = BR_TRANSFORM_MATRIX3T;
	BrMatrix3tTranslate(&actor_model->t.t.mat3t.mat, BR_SCALAR(-1.5), 0, 0);

	actor_model->model = model;
	
	actor_model = BrActorAdd(actor, BrActorAllocate(BR_ACTOR_MODEL, NULL));
	actor_model->t.type = BR_TRANSFORM_MATRIX3T;
	BrMatrix3tTranslate(&actor_model->t.t.mat3t.mat, BR_SCALAR(1.5), 0, 0);

	actor_model->model = BrModelAdd(BrModelClone(model));
	
	BrZsActorOrderTableSet(actor_model,
                BrZsOrderTableAllocate(127, BR_ORDER_TABLE_NEW_BOUNDS | BR_ORDER_TABLE_SORT_NEAR, BR_SORT_FIRST_VERTEX));

	/*
	 * set model RenderBoundsCallback fn
	 */
	BrZsRenderBoundsCallbackSet(Sample6_RenderBoundsCbfn);
}

/*
 * BRender Rendering function
 */
br_int_8 Sample6_RenderFn(void)
{
	br_uint_8 db_index = PSGfxDoubleBufferIndexRead();
    
        BrZsSceneRenderBegin(root, camera, back_screen[db_index]);

	BrZsSceneRenderAdd(actor);

	BrZsSceneRenderEnd();
}

/*
 * data Update function
 */
br_int_8 Sample6_UpdateFn(void)
{
	counter = (counter >= 360)?counter - 360:counter + 4;
		
	BrMatrix3tTranslate(&actor->t.t.mat3t.mat ,xpos, ypos, zpos);

	BrMatrix3tPre(&actor->t.t.mat3t.mat, &rotation);

	if(animate)
		BrMatrix3tPreRotateY(&actor->t.t.mat3t.mat, BR_ANGLE_DEG(counter));
}

/*
 * Pad test
 */
br_int_8 Sample6_PadTest(void)
{
	int ret = 0;

	unsigned long  padd = PadRead(1);

	if(PAD_COMPARE(padd, PAD_UP)) ypos = BR_ADD(ypos,BR_SCALAR(0.05));
	if(PAD_COMPARE(padd, PAD_DOWN)) ypos = BR_SUB(ypos,BR_SCALAR(0.05));

	if(PAD_COMPARE(padd, PAD_LEFT)) xpos = BR_SUB(xpos,BR_SCALAR(0.05));
	if(PAD_COMPARE(padd, PAD_RIGHT)) xpos = BR_ADD(xpos,BR_SCALAR(0.05));

	if(PAD_COMPARE(padd, PAD_TRI)) BrMatrix3tPreRotateX(&rotation,BR_ANGLE_DEG(2.0));
	if(PAD_COMPARE(padd, PAD_CROSS)) BrMatrix3tPreRotateX(&rotation,BR_ANGLE_DEG(-2.0));

	if(PAD_COMPARE(padd, PAD_FRONT_LTOP)) {
	    
	    	if(PAD_COMPARE(padd, PAD_SQUARE)) BrMatrix3tPreRotateZ(&rotation, BR_ANGLE_DEG(2.0));
		if(PAD_COMPARE(padd, PAD_CIRCLE)) BrMatrix3tPreRotateZ(&rotation, BR_ANGLE_DEG(-2.0));
	} else {

		if(PAD_COMPARE(padd, PAD_SQUARE)) BrMatrix3tPreRotateY(&rotation,BR_ANGLE_DEG(-2.0));
		if(PAD_COMPARE(padd, PAD_CIRCLE)) BrMatrix3tPreRotateY(&rotation,BR_ANGLE_DEG(2.0));
	}

	if(PAD_COMPARE(padd, PAD_FRONT_RTOP)) {
	    	zpos = BR_ADD(zpos,BR_SCALAR(0.5));
	}

	if(PAD_COMPARE(padd, PAD_FRONT_RBOT)) {
		zpos = BR_SUB(zpos,BR_SCALAR(0.5));
	}

	if(PAD_COMPARE(padd, PAD_FRONT_LBOT)) {
	    	animate ^= 1;

		while(PAD_COMPARE(PadRead(1), PAD_FRONT_LBOT));
	}

	return ret;
}

/*
 * Double buffer callback
 */
void BR_CALLBACK Sample6_DoubleBufferCbfn(br_uint_8 db_index)
{
    	int i;
	
    	/*
	 * use back_screen->height for correct NTSC/PAL height
	 */
        BrPixelmapRectangleCopy(back_screen[db_index], -back_screen[db_index]->origin_x, -back_screen[db_index]->origin_y, background, -background->origin_x, -background->origin_y, background->width, back_screen[db_index]->height);

	/*
	 * draw lines directly to screen
	 */
	for(i=0; i<bounding_box_count; i++) {
	    
		BrPixelmapLine(back_screen[db_index],
                        bounding_box[i][0]-1-back_screen[db_index]->origin_x,
                        bounding_box[i][1]-1-back_screen[db_index]->origin_y,
                        bounding_box[i][2]-back_screen[db_index]->origin_x,
                        bounding_box[i][1]-1-back_screen[db_index]->origin_y, outline);
		BrPixelmapLine(back_screen[db_index],
                        bounding_box[i][2]-back_screen[db_index]->origin_x,
                        bounding_box[i][1]-back_screen[db_index]->origin_y,
                        bounding_box[i][2]-back_screen[db_index]->origin_x,
                        bounding_box[i][3]-back_screen[db_index]->origin_y, outline);
		BrPixelmapLine(back_screen[db_index],
                        bounding_box[i][2]-back_screen[db_index]->origin_x,
                        bounding_box[i][3]-back_screen[db_index]->origin_y,
                        bounding_box[i][0]-back_screen[db_index]->origin_x,
                        bounding_box[i][3]-back_screen[db_index]->origin_y, outline);
                BrPixelmapLine(back_screen[db_index],
                        bounding_box[i][0]-1-back_screen[db_index]->origin_x,
                        bounding_box[i][3]-back_screen[db_index]->origin_y,
                        bounding_box[i][0]-1-back_screen[db_index]->origin_x,
                        bounding_box[i][1]-back_screen[db_index]->origin_y, outline);
	}
	bounding_box_count = 0;
}

/*
 * Model Renderbounds callback
 */
void BR_CALLBACK Sample6_RenderBoundsCbfn(
	br_actor *actor,
        const br_model *model,
        const br_material *material,
	br_order_table *order_table,
	br_uint_8 style,
        const br_matrix3t *model_to_screen,
        const br_int_32 bounds[4])
{
	br_uint_8 db_index = PSGfxDoubleBufferIndexRead();

	/*
	 * This bounding box is the model's 2D bounding volume.
	 * Always guaranteed to enclose rendered model.
	 */
	 
	/*
	 * generate background poly
	 */
	BrPSPrimPolyF4Set(&bounding_box_poly[bounding_box_count][db_index]);

	BrVector2Set(&bounding_box_poly[bounding_box_count][db_index].v0, bounds[0], bounds[1]);
	BrVector2Set(&bounding_box_poly[bounding_box_count][db_index].v1, bounds[2], bounds[1]);
	BrVector2Set(&bounding_box_poly[bounding_box_count][db_index].v2, bounds[0], bounds[3]);
	BrVector2Set(&bounding_box_poly[bounding_box_count][db_index].v3, bounds[2], bounds[3]);

	BrPSPrimColour0Set(&bounding_box_poly[bounding_box_count][db_index],
				BR_RED(fill[bounding_box_count]),
				BR_GRN(fill[bounding_box_count]),
				BR_BLU(fill[bounding_box_count]));

	BrPSPrimSemiTransSet(&bounding_box_poly[bounding_box_count][db_index], 1);

	/*
	 * add to start of current order table
	 */
	BrZsOrderTablePrimitiveInsert(order_table, &bounding_box_poly[bounding_box_count][db_index], order_table->size - 1);

	bounding_box[bounding_box_count][0] = bounds[0];
	bounding_box[bounding_box_count][1] = bounds[1];
	bounding_box[bounding_box_count][2] = bounds[2];
	bounding_box[bounding_box_count][3] = bounds[3];

	bounding_box_count++;
}

