/* Copyright (c) 1993-1996 Argonaut Technologies Limited. All rights reserved.
 *
 * $Id: SUPPORT.C 1.1 1996/09/12 14:01:22 NIKH Exp $
 * $Locker: $
 *
 * PS Generic test harness, common support functions
 */

/*
 * Sony library headers
 */
#include <sys/types.h>
#include <r3000.h>
#include <asm.h>
#include <kernel.h>
#include <libgte.h>
#include <libgpu.h>
#include <libetc.h>
#include <stdarg.h>

/*
 * BRender headers
 */
#include "brender.h"
#include "psio.h"

/*
 * test harness header
 */
#include "harness.h"

/*
 * generic support functions
 */
#include "support.h"


#define PRIMITIVE_HEAP  1024            /* BRender scratch workspace */

#define TILE_SIZE	256		/* tile size for textures */

/*
 * screen (VRAM device)
 */
br_pixelmap *screen;

/*
 * double buffered back screen
 */
br_pixelmap *back_screen[2];

/*
 * root of sample actor hierarchy
 */
br_actor *root;

/*
 * sample camera
 */
br_actor *camera;

/*
 * sample light
 */
br_actor *light;

/*
 * example file system and error handler
 */
extern br_filesystem PsyqFilesystem;
extern br_diaghandler PsyqDiagHandler;

extern br_uint_32 HeapBasePtr(void);
extern br_uint_32 _ramsize;
/*
 * generic setup of PS hardware, BRender, PSGfx
 */
br_int_8 _GenericInitDevice(void)
{
	br_material *mat;

	brps_draw_environment *draw;
	brps_display_environment *display;
	void *primitive_heap;
	int i;

#if 1
	/*
	 * set ramsize for blue/grey PlayStation
	 */
	_ramsize = 0x200000;
#else
	/*
	 * set ramsize for DTL-H2000
	 */
	_ramsize = 0x800000;
#endif

	/*
	 * Setup video mode
	 */
	InitHeap2(HeapBasePtr(), _ramsize - 0x800 - (HeapBasePtr() & 0xafffff));

#ifdef _NTSC
	SetVideoMode(MODE_NTSC);
#else
	SetVideoMode(MODE_PAL);
#endif

    	/*
	 * init BRender
	 */
	BrBegin();


	primitive_heap = BrMemAllocate(PRIMITIVE_HEAP, BR_MEMORY_APPLICATION);

	/*
	 * init PS Hardware
	 */
	PadInit(0);

	BrPSGeomInit();

	BrPSGraphReset(0);
	BrPSGraphDebugSet(0);

	/*
	 * setup double buffered environment
	 */
	draw = BrMemAllocate(sizeof(*draw) * 2, BR_MEMORY_APPLICATION);
	display = BrMemAllocate(sizeof(*display) * 2, BR_MEMORY_APPLICATION);

#ifdef _SAMPLE10
	BrPSDrawEnvironmentDefine(draw, 0, 0, SCREEN_WIDTH, SCREEN_HEIGHT);
	BrPSDrawEnvironmentDefine(draw +  1, 0, SCREEN_HEIGHT, SCREEN_WIDTH, SCREEN_HEIGHT);
	BrPSDisplayEnvironmentDefine(display, 0, SCREEN_HEIGHT, SCREEN_WIDTH, SCREEN_HEIGHT);
	BrPSDisplayEnvironmentDefine(display + 1, 0, 0, SCREEN_WIDTH, SCREEN_HEIGHT);
#else
	BrPSDrawEnvironmentDefine(draw, 0, 0, SCREEN_WIDTH, SCREEN_HEIGHT);
	BrPSDrawEnvironmentDefine(draw +  1, SCREEN_WIDTH, 0, SCREEN_WIDTH, SCREEN_HEIGHT);
	BrPSDisplayEnvironmentDefine(display, SCREEN_WIDTH, 0, SCREEN_WIDTH, SCREEN_HEIGHT);
	BrPSDisplayEnvironmentDefine(display + 1, 0, 0, SCREEN_WIDTH, SCREEN_HEIGHT);
#endif

	BrPSRectangleSet(&draw->texture_window, 0, 0, TILE_SIZE, TILE_SIZE);
	BrPSRectangleSet(&(draw+1)->texture_window, 0, 0, TILE_SIZE, TILE_SIZE);

#ifndef _NTSC
	BrPSRectangleSet(&display[0].screen, 0, 14, 256, 256);
	BrPSRectangleSet(&display[1].screen, 0, 14, 256, 256);
#endif

	display[0].interlace = 0;
	display[1].interlace = 0;

#ifndef _BACKGROUND
	draw[0].clear_screen = draw[1].clear_screen = 1;
#else
	draw[0].clear_screen = draw[1].clear_screen = 0;
#endif

	BrPSDrawEnvironmentRGBSet(draw, 0, 0, 0);
	BrPSDrawEnvironmentRGBSet(draw + 1, 0, 0, 0);

	screen = PSGfxBegin((void *)draw,				/* drawing environments */
			(void *)display,				/* display environments */
			BrMemAllocate(1024, BR_MEMORY_APPLICATION),	/* scratch buffer */
			1024,						/* scratch size */
			PS_GFX_DOUBLE_BUFFER);				/* flags */

	back_screen[0] = BrPixelmapMatch(screen, BR_PMMATCH_OFFSCREEN);
	back_screen[1] = BrPixelmapMatch(screen, BR_PMMATCH_OFFSCREEN);

	BrPSDisplayMaskSet(1);

	/*
	 * set double buffer callback function
	 */
	PSGfxDoubleBufferCallbackSet(_sample_fn_table->sample_double_buffer_cbfn);

	/*
	 * start up zsort renderer
	 */
	BrZsBegin(primitive_heap, PRIMITIVE_HEAP);

	/*
	 * init file system and error handler
	 */
	BrFilesystemSet(&PsyqFilesystem);
	BrDiagHandlerSet(&PsyqDiagHandler);

	/*
	 * init PlayStation -> PC host file system (SN)
	 */
	PCinit();

	/*
	 * setup generic data for samples
	 */
	root = BrActorAllocate(BR_ACTOR_NONE, NULL);
	root->identifier = "main_root";
	root->t.type = BR_TRANSFORM_MATRIX3T;
	BrMatrix3tIdentity(&root->t.t.mat3t.mat);

	camera = BrActorAdd(root, BrActorAllocate(BR_ACTOR_CAMERA, NULL));
	camera->identifier = "camera";
	((br_camera *)(camera->type_data))->type = BR_CAMERA_PERSPECTIVE;
	((br_camera *)(camera->type_data))->aspect = BR_SCALAR(1.21);
	((br_camera *)(camera->type_data))->hither_z = BR_SCALAR(0.05);
	((br_camera *)(camera->type_data))->yon_z = BR_SCALAR(100);

#ifdef _SAMPLE10
	camera->t.type = BR_TRANSFORM_LOOK_UP;
	BrVector3Set(&camera->t.t.look_up.up, 0, BR_SCALAR(1), 0);
	BrVector3Set(&camera->t.t.look_up.t, 0, 0, 0);
#else
	camera->t.type = BR_TRANSFORM_MATRIX3T;
	BrMatrix3tTranslate(&camera->t.t.mat3t.mat, 0, 0, BR_SCALAR(1));
#endif

	light = BrActorAdd(root, BrActorAllocate(BR_ACTOR_LIGHT, NULL));
	light->identifier = "main_light";
	light->t.type = BR_TRANSFORM_MATRIX3T;
	((br_light *)light->type_data)->colour = BR_COLOUR_RGB(255, 255, 255);
	BrMatrix3tIdentity(&light->t.t.mat3t.mat);

	BrLightEnable(light);

	/*
	 * setup order table at root
	 * (generate new bounds, use average of 3 vertices for sorting buckets)
	 */

#ifdef _SAMPLE10
	BrZsActorOrderTableSet(root,
                BrZsOrderTableAllocate(3, BR_ORDER_TABLE_NEW_BOUNDS | BR_ORDER_TABLE_SORT_NEAR, BR_SORT_FIRST_VERTEX));
#else
	BrZsActorOrderTableSet(root,
                BrZsOrderTableAllocate(1023, BR_ORDER_TABLE_NEW_BOUNDS | BR_ORDER_TABLE_SORT_NEAR, BR_SORT_MAX));
#endif

	FntLoad(960, 256);				/* font load */
	SetDumpFnt(FntOpen(32, 32, 320, 240, 0, 200)); /* stream open & define */

	/*
	 * return VSync mode
	 */
	return 0;
}

br_int_8 _GenericTerminate(void)
{
    	/*
	 * terminate Bucket Z Sort Renderer
	 */
    	BrZsEnd();

	/*
	 * terminate PSGfx support library
	 */
	PSGfxEnd();

	/*
	 * terminate BRender
	 */
	BrEnd();
}

br_int_8 _GenericLoopTest(void)
{
	unsigned long  padd = PadRead(1);
	br_int_8 ret = 0;

#if 0
	if(PAD_COMPARE(padd, PAD_SELECT)) {
		if(PAD_COMPARE(padd, PAD_START)) {
			ret = -1;
		}
	}
#endif

	return ret;
}

br_int_8 _GenericDoubleBuffer(void)
{
	/*
	 * swap environments, render display list
	 */
	BrPixelmapDoubleBuffer(screen, back_screen[PSGfxDoubleBufferIndexRead()]);
}

/*
 * This function is a nasty hack, and should be replaced with a real
 * GNU C vsprintf() function.  This function is not guaranteed to work under all
 * conditions, and is just a quick hack.
 * Sony do not currently supply a vsprintf function with their development libraries.
 * The following function will not be supported in any way.  Use it at your own risk...
 */
int vsprintf(char *buffer, const char *format,...)
{
	va_list args;
	char *scan, ch, new_format[10], *new_format_ptr;
	int found;

	/*
	 * get first argument
	 */
	va_start(args, format);
	args = (va_list)(br_uint_32 *)*(br_uint_32 *)args;

	for(scan = buffer;;) {

	    	/*
		 * scan for end of format string or '%'
		 */
		for(ch = *format; (ch != '\0') && (ch != '%'); ch = *(++format), scan++)
			*scan = ch;

		if(ch == '\0')
			break;

		new_format_ptr = new_format;

		*new_format_ptr++ = *format++;

		/*
		 * extract format for next argument
		 */
		for(found = 0; !found;) {

		    	ch = *new_format_ptr++ = *format++;

			/*
			 * Not all of these format flags are supported by the sprintf function
			 * supplied by Sony as a part of their development libraries.
			 */
		    	switch(ch) {
				case 's': case 'd': case 'i': case 'x': case 'X': case 'p': case 'c':
				case 'u': case 'e': case 'E': case 'f': case 'F': case 'g':
				case 'G': case 'n': case 'o':

					found = 1;
					break;
			}
		}

		/*
		 * terminate new format string
		 */
		*new_format_ptr = 0;

		new_format_ptr++;

		/*
		 * use sprintf to format current argument
		 */
		sprintf(scan, new_format, *(br_uint_32 *)args);

		/*
		 * update buffer pointer
		 */
		scan += strlen(scan);

		scan[0] = 0;
		scan[1] = 0;
		scan[2] = 0;
		scan[3] = 0;

		/*
		 * next argument (hack!)
		 */
		args += 4;
	}

	va_end(args);

	/*
	 * return length of formatted string
	 */
	return strlen(buffer);
}


br_uint_32 BR_CALLBACK ActorEnumCbfn(br_actor *mat, void *arg)
{
	(*(br_uint_32 *)arg)++;

    	return BrActorEnum(mat, ActorEnumCbfn, arg);
}


